(function () {
    'use strict';

    /* Shared modules */
    angular
        .module('app.core', [
            /* Cross-app modules */
            'app.config',
            'app.service',

            /* 3rd-party modules */
            'ionic'
        ]);

    /* Cross-app modules */
    angular
        .module('app.config', []);
    angular
        .module('app.service', []);

    /* Component modules */
    angular
        .module('app.component', []);

    /* Feature modules */
    angular
        .module('app.dashboard', []);
    angular
        .module('app.order', []);
    angular
        .module('app.user', []);
    angular
        .module('app.utils', []);


    /* Main module */
    angular
        .module('app', [
            /* Share modules */
            'app.core',

            /* Component modules */
            'app.component',

            /* Feature modules */
            'app.dashboard',
            'app.order',
            'app.user',
            'app.utils'
        ]);

    //////////

    angular
        .module('app')
        .run(runBlock);

    runBlock.$inject = [
        '$rootScope', '$ionicPlatform', '$ionicPopup', '$ionicLoading', '$ionicTabsDelegate',
        'routerStateService', 'authService', 'cartService'
    ];

    function runBlock($rootScope, $ionicPlatform, $ionicPopup, $ionicLoading, $ionicTabsDelegate,
                      routerStateService, authService, cartService) {
        $ionicPlatform.ready(function () {
            // Hide the accessory bar by default (remove this to show the
            // accessory bar above the keyboard for form inputs)
            if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            }
            if (window.StatusBar) {
                // org.apache.cordova.statusbar required
                StatusBar.styleLightContent();
            }
            if (navigator.splashscreen) {
                setTimeout(function () {
                    navigator.splashscreen.hide();
                }, 3000);
            }
        });

        $rootScope.$on('error:show', function (event, args) {
            if (!$rootScope.isAPIErrorAlertShowing) {
                $rootScope.isAPIErrorAlertShowing = true;
                var promise = $ionicPopup.alert({
                    title: args.message
                });
                promise.then(function () {
                    $rootScope.isAPIErrorAlertShowing = false;
                    if (args.nextState) {
                        routerStateService.go(args.nextState);
                    }
                });
            }
        });

        $rootScope.$on('loading:show', function () {
            if (!$rootScope.loadingOverlay) {
                $rootScope.loadingOverlay = $ionicLoading;
            }
            $rootScope.loadingOverlay.show();
        });
        $rootScope.$on('loading:hide', function () {
            if ($rootScope.loadingOverlay) {
                $rootScope.loadingOverlay.hide();
            }
        });

        $rootScope.$on('$stateChangeStart', function (event, nextState, nextParams, currentState) {
            var isAuthorized = authService.isAuthorized();
            if ('data' in nextState && 'requireLogin' in nextState.data && !isAuthorized) {
                event.preventDefault();
                routerStateService.go('login');
            } else if (isAuthorized && 'data' in nextState && 'noLoginRequired' in nextState.data) {
                event.preventDefault();
                $ionicTabsDelegate.select($ionicTabsDelegate.selectedIndex());
            }
        });

        cartService.refreshCart();
        $rootScope.$on('cartService:refreshCart', cartService.refreshCart);
    }
})();

(function () {
    'use strict';

    angular
        .module('app')
        .config(configure);

    configure.$inject = ['$stateProvider', '$urlRouterProvider', '$httpProvider', '$ionicConfigProvider'];

    function configure($stateProvider, $urlRouterProvider, $httpProvider, $ionicConfigProvider) {
        // Ionic uses AngularUI Router which uses the concept of states
        // Learn more here: https://github.com/angular-ui/ui-router
        // Set up the various states which the app can be in.
        // Each state's controller can be found in controllers.js
        $stateProvider
            // setup an abstract state for the tabs directive
            .state('tab', {
                url: '/tab',
                abstract: true,
                templateUrl: 'templates/layout/tabs.html'
            })

            // Each tab has its own nav history stack
            // Dashboard
            .state('tab.dashboard', {
                url: '/dashboard',
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/dashboard.html'
                    }
                }
            })
            .state('dashboard-locations', {
                url: '/locations',
                parent: 'tab',
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/locations.html'
                    }
                }
            })
            .state('dashboard-product', {
                url: '/product/:id',
                parent: 'tab',
                cache: false,
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/product.html'
                    }
                }
            })
            .state('dashboard-product-types', {
                url: '/product-types',
                parent: 'tab',
                cache: false,
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/product-types.html'
                    }
                }
            })
            .state('dashboard-product-list', {
                cache: false,
                url: '/product-list',
                parent: 'tab',
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/product-list.html'
                    }
                }
            })
            .state('dashboard-checkout', {
                url: '/checkout',
                parent: 'tab',
                params: {
                    'products': null,
                    'isDirectBuy': false
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/checkout.html'
                    }
                }
            })
            .state('dashboard-order-list', {
                cache: false,
                url: '/order-list',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/order-list.html'
                    }
                }
            })
            .state('dashboard-order-info', {
                cache: false,
                url: '/order-info',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/order-info.html'
                    }
                }
            })
            .state('dashboard-order-refund', {
                url: '/order-refund',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/order-refund.html'
                    }
                }
            })
            .state('dashboard-pay', {
                url: '/pay/:transId',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/pay.html'
                    }
                }
            })
            .state('dashboard-pay-success', {
                url: '/pay-success',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/pay-success.html'
                    }
                }
            })
            .state('dashboard-user-address-list', {
                url: '/user-address-list',
                parent: 'tab',
                params: {
                    'address_id': '',
                    'products': null
                },
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/user-address-list.html'
                    }
                }
            })
            .state('dashboard-user-address', {
                url: '/user-address',
                parent: 'tab',
                params: {
                    'address': null,
                    'type': 'dashboard'
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/user-address.html'
                    }
                }
            })
            .state('dashboard-login', {
                url: '/login',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/login.html'
                    }
                }
            })
            .state('dashboard-register', {
                url: '/register',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/register.html'
                    }
                }
            })
            .state('dashboard-retrieve-password', {
                url: '/retrieve-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/retrieve-password.html'
                    }
                }
            })
            .state('dashboard-reset-password', {
                url: '/reset-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                params: {token: ''},
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/reset-password.html'
                    }
                }
            })
            .state('dashboard-custom-invoice', {
                url: '/custom-invoice',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/custom-invoice.html'
                    }
                }
            })

            // Cart
            .state('tab.cart', {
                url: '/cart',
                cache: false,
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/cart.html'
                    }
                }
            })
            .state('cart-product', {
                url: '/product/:id',
                parent: 'tab',
                cache: false,
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/product.html'
                    }
                }
            })
            .state('cart-checkout', {
                url: '/checkout',
                parent: 'tab',
                params: {'products': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/checkout.html'
                    }
                }
            })
            .state('cart-order-list', {
                cache: false,
                url: '/order-list',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/order-list.html'
                    }
                }
            })
            .state('cart-order-info', {
                cache: false,
                url: '/order-info',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/order-info.html'
                    }
                }
            })
            .state('cart-order-refund', {
                url: '/order-refund',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/order-refund.html'
                    }
                }
            })
            .state('cart-pay', {
                url: '/pay/:transId',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/pay.html'
                    }
                }
            })
            .state('cart-pay-success', {
                url: '/pay-success',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/pay-success.html'
                    }
                }
            })
            .state('cart-user-address-list', {
                url: '/user-address-list',
                parent: 'tab',
                params: {
                    'address_id': '',
                    'products': null
                },
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/user-address-list.html'
                    }
                }
            })
            .state('cart-user-address', {
                url: '/cart-user-address',
                parent: 'tab',
                params: {
                    'address': null,
                    'type': 'cart'
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/user-address.html'
                    }
                }
            })
            .state('cart-login', {
                url: '/login',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/login.html'
                    }
                }
            })
            .state('cart-register', {
                url: '/register',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/register.html'
                    }
                }
            })
            .state('cart-retrieve-password', {
                url: '/retrieve-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/retrieve-password.html'
                    }
                }
            })
            .state('cart-reset-password', {
                url: '/reset-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                params: {token: ''},
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/reset-password.html'
                    }
                }
            })
            .state('cart-custom-invoice', {
                url: '/custom-invoice',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/custom-invoice.html'
                    }
                }
            })

            // Account
            .state('tab.account', {
                url: '/account',
                cache: false,
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/account.html'
                    }
                }
            })
            .state('account-settings', {
                url: '/settings',
                parent: 'tab',
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/settings.html',
                        controller: 'SettingsController as settings'
                    }
                }
            })
            .state('account-login', {
                url: '/login',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/login.html'
                    }
                }
            })
            .state('account-retrieve-password', {
                url: '/retrieve-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/retrieve-password.html'
                    }
                }
            })
            .state('account-reset-password', {
                url: '/reset-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                params: {token: ''},
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/reset-password.html'
                    }
                }
            })
            .state('account-register', {
                url: '/register',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/register.html'
                    }
                }
            })
            .state('account-user-address-list', {
                url: '/user-address-list',
                parent: 'tab',
                params: {'type': 'account'},
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/user-address-list.html'
                    }
                }
            })
            .state('account-user-address', {
                url: '/user-address',
                parent: 'tab',
                params: {
                    'address': null
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/user-address.html'
                    }
                }
            })
            .state('account-user-credits', {
                url: '/get-balance',
                parent: 'tab',
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/user-credits.html'
                    }
                }
            })
            .state('account-charge-redemption', {
                url: '/charge-redemption',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/charge-redemption.html'
                    }
                }
            })
            .state('account-userinfo', {
                url: '/userinfo',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/user-info.html'
                    }
                }
            })
            .state('account-security-settings', {
                url: '/security-settings',
                parent: 'tab',
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/security-settings.html'
                    }
                }
            })
            .state('account-change-password', {
                url: '/change-password',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/change-password.html'
                    }
                }
            })
            .state('account-order-list', {
                cache: false,
                url: '/order-list',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/order-list.html'
                    }
                }
            })
            .state('account-order-info', {
                cache: false,
                url: '/order-info',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/order-info.html'
                    }
                }
            })
            .state('account-order-refund', {
                url: '/order-refund',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/order-refund.html'
                    }
                }
            })
            .state('account-pay', {
                url: '/pay/:transId',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/pay.html'
                    }
                }
            })
            .state('account-pay-success', {
                url: '/pay-success',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/pay-success.html'
                    }
                }
            })
            .state('account-bind-strategy', {
                url: '/bind-strategy',
                parent: 'tab',
                cache: false,
                params: {
                    'type': '',
                    'typeValue': '',
                    'isBind': false
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/bind-strategy.html'
                    }
                }
            });


        // if none of the above states are matched, use this as the fallback
        $urlRouterProvider.otherwise('/tab/dashboard');

        // Register http interceptor
        if (ArkenstoneAppSettings.debug) {
            $httpProvider.defaults.headers.common['X-Requested-With'] = 'XMLHttpRequest';
        }
        $httpProvider.defaults.withCredentials = true;
        $httpProvider.interceptors.push('httpInterceptor');

        // Platform configurations
        $ionicConfigProvider.backButton.text('').icon('ion-ios-arrow-left').previousTitleText(false);
        $ionicConfigProvider.navBar.alignTitle('center');
        $ionicConfigProvider.tabs.position('bottom').style('standard');
    }
})();

(function() {
    'use strict';

    angular
        .module('app.config')
        .constant('appConf', {
            maxItemsDisplayedInList: 20,
            maxRatingStarValue: 5,
            disableAddToCartBtnInProductList: true,
            enableNavCategorySlider: false,
            iosAppStoreUrl: 'itms-apps://itunes.apple.com/cn/app/wo-men-de-can-zhuo/id1035564204?l=zh&ls=1&mt=8'
        })
        .constant('appConst', {
            USER_GENDER: {
                UNKNOWN: 0,
                MALE: 1,
                FEMALE: 2
            },
            ORDER_STATUS: {
                WAITING_FOR_PAYMENT: 1,
                WAITING_FOR_SHIPMENT: 2,
                SHIPPING: 3,
                COMPLETED: 4,
                CANCELLED: 5,
                REFUND_REQUESTED: 6,
                REFUNDED: 7
            }
        });
})();

ArkenstoneAppSettings = {
    baseApiUrl: 'http://www.ourdts.com/api/',
    debug: false,
    gaodeApiKey: 'eb5957712c72c5b08ea3af04e04d5d00',
    placeholder: null,
    gaodeScriptUrl: 'http://webapi.amap.com/maps?v=1.3'
};

(function () {
    'use strict';

    angular
        .module('app.component')
        .directive('arkCartTabWatcher', arkCartTabWatcher);

    arkCartTabWatcher.$inject = ['$rootScope', 'cartService'];

    function arkCartTabWatcher($rootScope, cartService) {
        var directive = {
            link: link,
            restrict: 'A'
        };
        return directive;

        function link(scope, elem, attrs) {
            $rootScope.$on('cartService:refreshCountInCart', function () {
                scope.$$childHead.badge = cartService.getTotalQuantity();
            });
        }
    }
})();

(function() {
    angular
        .module('app.component')
        .directive('fileModel', fileModel);

        fileModel.$inject = ['alertService'];

        function fileModel(alertService) {
            var directive = {
                restrict: 'A',
                require: '?ngModel',
                link: link
            };
            return directive;

            function link(scope, element, attrs, ngModel) {
                element.bind('change', function(event) {
                    if (event.target.files[0].size >= 5000000) {
                        // the picture size should not more than 5M
                        alertService.autoShow('每张图片大小不超过5M，请重新上传');
                        $(event.target).val('');
                        return;
                    }
                    ngModel.$setViewValue(event.target.files[0]);
                    $(event.target).next('img#' + attrs.id + '-img').attr('src', URL.createObjectURL(event.target.files[0]));
                    //$(event.target).next('img#' + attrs.id + '-img').css('display', 'block');
                    $(event.target).next('img#' + attrs.id + '-img').show();
                });
            }
        }
})();

(function() {
    angular
        .module('app.component')
        .directive('arkNumberSelector', arkNumberSelector);

    function arkNumberSelector() {
        var directive = {
            restrict: 'E',
            replace: true,
            templateUrl: 'templates/partials/number-selector.directive.html',
            link: link,
            scope: {
                value: '@',
                maxValue: '@',
                minValue: '@',
                disabledMinus: '@',
                disabledPlus: '@',
                changeEvent: '&'
            }
        };
        return directive;

        function link(scope, element, attrs) {
            scope.$watch('minValue', function () {
                if (!isNaN(parseInt(scope.minValue, 10))) {
                    scope.value = Math.max(scope.value, scope.minValue);
                    scope.minValue = parseInt(scope.minValue, 10);
                }
            });

            scope.$watch('maxValue', function () {
                if (!isNaN(parseInt(scope.maxValue, 10))) {
                    scope.value = Math.min(scope.value, scope.maxValue);
                    scope.maxValue = parseInt(scope.maxValue, 10);
                }
            });

            scope.$watch('value', function () {
                scope.disabledMinus = scope.value === scope.minValue;
                scope.disabledPlus = scope.value === scope.maxValue;
            });

            scope.minusNum = function() {
                if (--scope.value < scope.minValue) {
                    scope.value++;
                }
            };

            scope.plusNum = function() {
                if (++scope.value > scope.maxValue) {
                    scope.value--;
                }
            };
        }
    }
})();

(function () {
    angular
        .module('app.component')
        .directive('arkProductItem', arkProductItem);

    arkProductItem.$inject = ['routerStateService', 'cartService'];

    function arkProductItem(routerStateService, cartService) {
        var directive = {
            restrict: 'E',
            replace: true,
            templateUrl: 'templates/partials/product-item.directive.html',
            link: link,
            scope: {
                brief: '@',
                flyToCart: '&',
                goProductDetail: '&',
                id: '@',
                imgUrl: '@',
                leftQuantity: '@',
                priceText: '@',
                showCartBtn: '@',
                title: '@'
            }
        };
        return directive;

        function link(scope, element, attrs) {
            scope.flyToCart = function () {
                cartService.flyToCart(scope.imgUrl);
                cartService.addToCart(scope.id, 1);
            };

            scope.goProductDetail = function () {
                routerStateService.go('product', {id: scope.id});
            };
        }
    }
})();

(function() {
    'use strict';

    angular
        .module('app.component')
        .directive('arkScrollToTop', arkScrollToTop);

    arkScrollToTop.$inject = ['$ionicScrollDelegate', '$rootScope'];

    function arkScrollToTop($ionicScrollDelegate, $rootScope) {
        var directive = {
            link: link,
            replace: true,
            restrict: 'E',
            scope: {
                scrollToTop: '&'
            },
            templateUrl: 'templates/partials/scroll-to-top.directive.html'
        };
        return directive;

        function link(scope, elem, attrs) {
            scope.scrollToTop = scrollToTop;

            function scrollToTop() {
                $rootScope.$broadcast('scrollTop');
                $ionicScrollDelegate.scrollTop(true);
            }
        }
    }
})();

(function() {
    'use strict';

    angular
        .module('app.component')
        .directive('arkScrollWatch', arkScrollWatch);

    arkScrollWatch.$inject = ['$rootScope', '$ionicScrollDelegate'];

    function arkScrollWatch($rootScope, $ionicScrollDelegate) {
        var directive = {
            link: link,
            restrict: 'A'
        };
        return directive;

        function link(scope, elem, attrs) {
            elem.bind('scroll', function() {
                if ($ionicScrollDelegate.getScrollPosition().top > 300) {
                    $('.scroll-to-top').fadeIn();
                } else {
                    $('.scroll-to-top').fadeOut();
                }
            });

            $rootScope.$on('$ionicView.leave', function() {
                $('.scroll-to-top').hide();
            });
        }
    }
})();

(function() {
    angular
        .module('app.component')
        .directive('arkStarRating', arkStarRating);

    arkStarRating.$inject = ['appConf'];

    function arkStarRating(appConf) {
        var directive = {
            restrict: 'E',
            replace: true,
            templateUrl: 'templates/partials/star-rating.directive.html',
            link: link,
            require: '?ngModel',
            scope: {
                maxValue: '@',
                ratingValue: '=ngModel',
                updateStar: '&'
            }
        };
        return directive;

        function link(scope, element, attrs, ngModel) {
            if (scope.maxValue === undefined) {
                scope.maxValue = appConf.maxRatingStarValue;
            }

            scope.$watch('ratingValue', function() {
                scope.stars = [];
                for (var i = 0; i < scope.maxValue; i++) {
                    scope.stars.push({
                        'fa-star-half-o': scope.ratingValue > i && scope.ratingValue < i + 1,
                        'fa-star': scope.ratingValue >= i + 1,
                        'fa-star i-unselected': scope.ratingValue <= i
                    });
                }
            });

            scope.updateStar = function(index, set) {
                if (attrs.read) {
                    return;
                }
                updateItem(index);
                if (set) {
                    ngModel.$setViewValue(parseInt(index) + 1);
                }
            };

            function updateItem(index) {
                if (index === -1) {
                    index = ngModel.$viewValue - 1;
                }
                $(element.find('i')).each(function(i) {
                    if (i <= index) {
                        $(this).removeClass('fa fa-star i-unselected').addClass('fa fa-star');
                    } else if (i > index) {
                        $(this).addClass('fa fa-star i-unselected');
                    }
                });
            }
        }
    }
})();

(function() {
    'use strict';

    angular
        .module('app.component')
        .directive('arkTabsWatcher', arkTabsWatcher);

    arkTabsWatcher.$inject = ['$rootScope'];

    function arkTabsWatcher($rootScope) {
        var directive = {
            link: link,
            restrict: 'A'
        };
        return directive;

        function link(scope, elem, attrs) {
            $rootScope.$on('$ionicView.beforeEnter', function(e, view) {
                if (view.stateName && view.stateName.indexOf('tab.') === -1) {
                    $(elem).addClass('tabs-item-hide');
                } else {
                    $(elem).removeClass('tabs-item-hide');
                }
            });
        }
    }
})();

(function() {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('CategoryNavController', CategoryNavController);

    CategoryNavController.$inject = ['httpService', '$ionicSlideBoxDelegate', 'alertService'];

    function CategoryNavController(httpService, $ionicSlideBoxDelegate, alertService) {
        var vm = this;
        vm.categoryGroups = [];
        vm.maxCountPerPage = 4;
        vm.onCategoryClick = onCategoryClick;

        fetchCategories();

        ////////////////

        function fetchCategories() {
            var promise = httpService.get('category_list');
            promise.then(function(result) {
                var categories = result.data.categoryList;
                if (categories) {
                    while (categories.length) {
                        vm.categoryGroups.push(categories.splice(0, vm.maxCountPerPage));
                    }
                    $ionicSlideBoxDelegate.update();
                }
            });
        }

        function onCategoryClick(category) {
            alertService.autoShow(category.name);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('ProductDetailController', ProductDetailController);

    ProductDetailController.$inject = [
        '$scope', 'httpService', '$stateParams', 'routerStateService',
        '$ionicScrollDelegate', '$timeout', '$ionicHistory', 'cartService',
        'authService', 'modalManagerService', 'alertService'
    ];

    function ProductDetailController($scope, httpService, $stateParams, routerStateService,
                                     $ionicScrollDelegate, $timeout, $ionicHistory, cartService,
                                     authService, modalManagerService, alertService) {
        var vm = this;
        vm.activeDetailTab = 'detail';
        vm.addToCart = addToCart;
        vm.cartSize = cartService.getTotalQuantity();
        vm.changeNumber = changeNumber;
        vm.deliveryCtrl = null;
        vm.deliveryModal = null;
        vm.detailImages = [];
        vm.hasMore = true;
        vm.loadDetails = loadDetails;
        vm.loadBasicInfo = loadBasicInfo;
        vm.onCheckout = onCheckout;
        vm.onDeliveryAreaClick = onDeliveryAreaClick;
        vm.product = null;
        vm.productReviews = [];
        vm.quantity = 1;
        vm.questionList = [];
        vm.refreshProgress = refreshProgress;
        vm.showQuestionBtn = authService.isAuthorized();
        vm.switchDetailTab = switchDetailTab;


        fetchProduct();
        $scope.$on('scrollTop', loadBasicInfo);

        ////////////////

        function addToCart() {
            cartService.addToCart(vm.product.id, vm.quantity, function () {
                vm.cartSize = cartService.getTotalQuantity();
                alertService.autoShow('恭喜，已添加至购物车。');
            });
        }

        function changeNumber(value) {
            vm.quantity = value;
        }

        function fetchProduct() {
            var promise = httpService.get('product/' + $stateParams.id);
            promise.then(function (result) {
                vm.product = result.data.product;
            }, function (error) {
                if (error.status === 404) {
                    $ionicHistory.goBack();
                }
            });
        }

        function loadBasicInfo() {
            $scope.$broadcast('scroll.refreshComplete');
            vm.hasMore = true;
            $ionicScrollDelegate.scrollTop();
        }

        function loadDetails() {
            if (vm.detailImages.length) {
                showDetails();
            } else {
                var promise = httpService.get('product_details/' + $stateParams.id);
                promise.then(function (result) {
                    vm.detailImages = result.data.detailImages;
                    showDetails();
                });
            }
        }

        function onCheckout() {
            var product = angular.extend({}, {
                quantity: vm.quantity,
                checked: true
            }, vm.product);
            routerStateService.go('checkout', {
                products: [product],
                isDirectBuy: true
            });
        }

        function onDeliveryAreaClick() {
            $scope.defaultArea = {
                province: {
                    id: vm.product.deliveryRange.provinceID,
                    name: vm.product.deliveryRange.provinceName
                },
                city: {
                    id: vm.product.deliveryRange.cityID,
                    name: vm.product.deliveryRange.cityName
                },
                district: {
                    id: vm.product.deliveryRange.districtID,
                    name: vm.product.deliveryRange.districtName
                }
            };
            $scope.selectedArea = null;
            $scope.modalTitle = '配送至';
            modalManagerService.showModal('productDeliveryArea', $scope, {
                onHidden: function (modal) {
                    if (!modal.scope.selectedArea) {
                        return;
                    }
                    var promise = httpService.get('product/' + vm.product.id + '/delivery_range/' + modal.scope.selectedArea.district.id);
                    promise.then(function (result) {
                        vm.product.deliveryRange = result.data.deliveryRange;
                        modal.scope.selectedArea = null;
                    });
                }
            });
        }

        function refreshProgress() {
            if (vm.product) {
                return {
                    width: vm.product.statsInfo.progress + '%'
                };
            }
        }

        function showDetails() {
            $scope.$broadcast('scroll.infiniteScrollComplete');
            vm.hasMore = false;
            $ionicScrollDelegate.scrollTop();
        }

        function showQuestionsAndAnswers() {
            if (!vm.questionList.length) {
                var promise = httpService.get('product_faq_list/' + $stateParams.id);
                promise.then(function (result) {
                    vm.questionList = result.data.faqList;
                });
            }
        }

        function showReviews() {
            if (!vm.productReviews.length) {
                var promise = httpService.get('product_review_list/' + $stateParams.id);
                promise.then(function (result) {
                    vm.productReviews = result.data.reviewList;
                });
            }
        }

        function switchDetailTab(tabType) {
            $ionicScrollDelegate.scrollTop();
            if (vm.activeDetailTab === tabType) {
                return;
            }
            vm.activeDetailTab = tabType;
            if (tabType === 'review') {
                showReviews();
            } else if (tabType === 'qa') {
                showQuestionsAndAnswers();
            } else {
                loadDetails();
            }
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('ProductListController', ProductListController);

    ProductListController.$inject = [
        '$scope', 'appConf', 'routerStateService', '$ionicViewSwitcher',
        '$ionicScrollDelegate', 'productSearchService'
    ];

    function ProductListController($scope, appConf, routerStateService, $ionicViewSwitcher,
                                   $ionicScrollDelegate, productSearchService) {
        var vm = this;
        vm.disableAddToCartBtnInProductList = appConf.disableAddToCartBtnInProductList;
        vm.hasMore = true;
        vm.page = 0;
        vm.productList = [];
        vm.orderBy = productSearchService.getParam('orderBy');
        vm.keyword = productSearchService.getParam('keyword');
        vm.goBack = goBack;
        vm.getProducts = getProducts;
        vm.queryByKeyword = queryByKeyword;
        vm.sortProducts = sortProducts;

        ////////////////

        function goBack() {
            productSearchService.resetParams();
            $ionicViewSwitcher.nextDirection('back');
            routerStateService.go('tab.dashboard');
        }

        function getProducts() {
            if (!vm.hasMore) {
                return;
            }

            var promise = productSearchService.search(vm.page);
            promise.then(function (result) {
                vm.productList = vm.productList.concat(result.data.productList);
                vm.hasMore = result.data.hasNextPage;
                vm.page = result.data.nextPage;
            }).catch(function () {
                vm.hasMore = false;
            }).finally(function () {
                $scope.$broadcast('scroll.infiniteScrollComplete');
            });
        }

        function queryByKeyword() {
            if (!vm.keyword || vm.keyword === productSearchService.getParam('keyword')) {
                return;
            }
            productSearchService.resetParams();
            productSearchService.setParam('keyword', vm.keyword);
            vm.orderBy = ['newest', true];
            if (routerStateService.getCurrentName().indexOf('product-list') === -1) {
                $ionicViewSwitcher.nextDirection('forward');
                routerStateService.go('product-list');
            } else {
                resetState();
            }
        }

        function resetState() {
            vm.productList = [];
            vm.page = 0;
            vm.hasMore = true;
            vm.orderBy = productSearchService.getParam('orderBy');
            vm.keyword = productSearchService.getParam('keyword');
            $ionicScrollDelegate.scrollTop();
        }

        function sortProducts(condition) {
            if (condition === vm.orderBy[0]) {
                vm.orderBy[1] = !vm.orderBy[1];
            } else {
                if (condition === 'newest') {
                    vm.orderBy = ['newest', true];
                } else {
                    vm.orderBy = ['left_quantity', false];
                }
            }
            productSearchService.setParam('orderBy', vm.orderBy);
            resetState();
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.order')
        .controller('ProductTypeController', ProductTypeController);

    ProductTypeController.$inject = ['httpService', '$ionicViewSwitcher', 'routerStateService', 'productSearchService'];

    function ProductTypeController(httpService, $ionicViewSwitcher, routerStateService, productSearchService) {
        var vm = this;
        vm.selectedType = productSearchService.getParam('productType');
        vm.getProductTypes = getProductTypes;
        vm.goToProductList = goToProductList;
        vm.productTypes = [{name: '全部', code: 0}];

        getProductTypes();

        ////////////////

        function getProductTypes() {
            var productTypes = httpService.get('product_types');
            productTypes.then(function (result) {
                vm.productTypes = vm.productTypes.concat(result.data.productTypes);
            });
        }

        function goToProductList(type) {
            productSearchService.setParam('productType', type.code);
            $ionicViewSwitcher.nextDirection('back');
            routerStateService.go('product-list');
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('QuestionController', QuestionController);

    QuestionController.$inject = ['$scope', 'httpService'];

    function QuestionController($scope, httpService) {
        var vm = this;
        vm.msg = null;
        vm.sendQuestion = sendQuestion;

        ////////////////

        function sendQuestion() {
            if (vm.msg == null || vm.msg.length <= 0) {
                return;
            }
            var post_data = {
                'product_id': $scope.productID,
                'question': vm.msg
            };
            var promise = httpService.post('user/add_question', post_data);
            promise.then(function (result) {
                if (result.data.success === true) {
                    vm.msg = null;
                }
            });
        }
    }

})();

(function () {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('RecommendProductController', RecommendProductController);

    RecommendProductController.$inject = ['$rootScope', 'productSearchService', 'appConf'];

    function RecommendProductController($rootScope, productSearchService, appConf) {
        var vm = this;
        vm.disableAddToCartBtnInProductList = appConf.disableAddToCartBtnInProductList;
        vm.numLimit = appConf.maxItemsDisplayedInList;
        vm.products = [];

        fetchProducts();
        // refresh products according to current city
        $rootScope.$on('currentCity:changed', fetchProducts);

        ////////////////

        function fetchProducts() {
            var promise = productSearchService.search();
            promise.then(function (result) {
                vm.products = result.data.productList;
            });
        }
    }
})();

(function() {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('SpecialController', SpecialController);

    SpecialController.$inject = [
        '$rootScope', '$scope', 'httpService', 'routerStateService',
        '$ionicSlideBoxDelegate', 'currentCityService'];

    function SpecialController(
        $rootScope, $scope, httpService, routerStateService, $ionicSlideBoxDelegate, currentCityService) {
        var vm = this;
        vm.specialGroups = [];
        vm.onSpecialClick = onSpecialClick;

        fetchSpecial();
        // refresh products according to current city
        $rootScope.$on('currentCity:changed', fetchSpecial);
        // if vm.specialGroups is updated when this is not the current state,
        // $ionicSlideBoxDelegate.update doesn't work since it cannot know the
        // right size of the view. This ensures update is always called when
        // this becomes the current view.
        $scope.$on('$stateChangeSuccess', update);

        ////////////////

        function fetchSpecial() {
            var params = {};
            var city = currentCityService.getCurrentCity();
            if (city && city.id) {
                params.current_city = city.id;
            }
            var promise = httpService.get('recommend_product_list', params);
            promise.then(function(result) {
                vm.specialGroups = result.data.recommendProductList;
                update();
            });
        }

        function update(event, toState, toParams, fromState, fromParams) {
            if (!toState || toState.name === 'tab.dashboard') {
                $ionicSlideBoxDelegate.update();
                $ionicSlideBoxDelegate.slide(0);
                $ionicSlideBoxDelegate.start();
            }
        }

        function onSpecialClick(special) {
            routerStateService.go('product', {'id': special.productID});
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.order')
        .controller('CartController', CartController);

    CartController.$inject = ['$ionicPopup', 'routerStateService', 'cartService', 'alertService'];

    function CartController($ionicPopup, routerStateService, cartService, alertService) {
        var vm = this;
        vm.changeQuantity = changeQuantity;
        vm.checkAllProducts = checkAllProducts;
        vm.checkProduct = checkProduct;
        vm.deleteAllProducts = deleteAllProducts;
        vm.deleteProduct = deleteProduct;
        vm.goToProduct = goToProduct;
        vm.onCheckout = onCheckout;
        vm.hasAnyProduct = false;
        vm.isAllChecked = false;
        vm.isAnyChecked = false;
        vm.products = [];
        vm.totalPrice = 0;

        getProducts();

        ////////////////

        function changeQuantity(product, quantity) {
            if (product.quantity !== quantity) {
                cartService.updateCart(product.id, quantity, product.checked, getProducts);
            }
            event.cancelBubble = true;
        }

        function checkAllProducts() {
            var products = [];
            angular.forEach(vm.products, function (product) {
                products.push({
                    sale: product.id,
                    quantity: product.quantity,
                    checked: vm.isAllChecked
                });
            });
            cartService.bulkUpdateCart(products, getProducts);
        }

        function checkProduct(product) {
            cartService.updateCart(product.id, product.quantity, product.checked, getProducts);
            event.cancelBubble = true;
        }

        function deleteAllProducts() {
            var performDeleteAll = function () {
                var ids = [];
                angular.forEach(vm.products, function (product) {
                    if (product.checked) {
                        ids.push(product.id);
                    }
                });
                cartService.bulkRemoveFromCart(ids, getProducts);
            };

            $ionicPopup.confirm({
                title: '确认删除',
                template: '你确认要删除购物车里面选中的商品？',
                buttons: [
                    {
                        text: '删除',
                        type: 'button-assertive',
                        onTap: performDeleteAll
                    }, {
                        text: '取消',
                        type: 'button-balanced'
                    }
                ]
            });
        }

        function deleteProduct(product) {
            cartService.removeFromCart(product.id, getProducts);
        }

        function getProducts() {
            vm.products = cartService.getProductsInCart();
            refreshStatus();
        }

        function refreshStatus() {
            var total = 0;
            var checked = vm.products.length ? true : false;
            vm.isAnyChecked = false;
            vm.hasAnyProduct = vm.products.length > 0;
            angular.forEach(vm.products, function (product) {
                if (!product.statsInfo.completed) {
                    if (product.checked) {
                        total += product.quantity * product.price;
                        vm.isAnyChecked = true;
                    } else {
                        checked = false;
                    }
                }
            });
            vm.totalPrice = total;
            vm.isAllChecked = checked;
        }

        function onCheckout() {
            var selectProducts = [];
            var valid = true;
            angular.forEach(vm.products, function (product) {
                if (product.checked) {
                    if (product.statsInfo.completed) {
                        refreshStatus();
                        valid = false;
                    } else {
                        selectProducts.push(product);
                    }
                }
            });

            if (selectProducts.length > 0 && valid) {
                routerStateService.go('checkout', {'products': selectProducts});
            } else if (!valid) {
                alertService.autoShow('购物车中存在失效商品，请删除后重新结算');
            }
        }

        function goToProduct(item) {
            if (!item.statsInfo.completed) {
                routerStateService.go('product', {
                    id: item.id
                });
            }
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.order')
        .controller('CheckoutController', CheckoutController);

    CheckoutController.$inject = ['httpService', '$stateParams', '$scope', 'routerStateService', 'cartService'];

    function CheckoutController(httpService, $stateParams, $scope, routerStateService, cartService) {
        var vm = this;
        vm.address = null;
        vm.invoiceTitle = null;
        vm.canSubmit = false;
        vm.products = [];
        vm.totalPrice = 0;
        vm.subTotal = 0;
        vm.availablePoints = 0;
        vm.balance = 0;
        vm.pointsRate = 100;
        vm.usePoints = false;
        vm.showPoints = false;
        vm.goAddress = goAddress;
        vm.goInvoice = goInvoice;
        vm.submitOrder = submitOrder;
        vm.updateProduct = updateProduct;

        $scope.$on('checkout:refreshAddress', refreshAddress);
        $scope.$on('checkout:refreshInvoice', refreshInvoice);

        readInfo();

        //////////////////////////

        function goAddress() {
            var addressId = '';
            if (vm.address) {
                addressId = vm.address.id;
            }
            routerStateService.go('user-address-list', {
                address_id: addressId,
                products: vm.products
            });
        }

        function goInvoice() {
            routerStateService.go('custom-invoice');
        }

        function readInfo() {
            var promise = httpService.get('user/default_address');
            promise.then(function (result) {
                if (result.data.defaultAddress.id) {
                    vm.address = result.data.defaultAddress;
                }
                updateProduct();
            });
            httpService.get('get_balance').then(function (result) {
                vm.availablePoints = result.data.points;
                vm.pointsRate = result.data.rate;
                vm.balance = result.data.balance;
            });
        }

        function updateProduct(products) {
            vm.products = products || $stateParams.products || [];
            vm.canSubmit = vm.address != null;
            vm.subTotal = 0;
            angular.forEach(vm.products, function (product) {
                vm.subTotal += product.price * product.quantity;
                product.unreachable = true;
                if (vm.address) {
                    deliveryRange(product, vm.address.districtID);
                }
            });
            vm.totalPrice = vm.subTotal;
            if (vm.usePoints) {
                vm.totalPrice -= Math.min(vm.availablePoints / vm.pointsRate, vm.totalPrice);
            }
        }

        function deliveryRange(product, districtID) {
            var promise = httpService.get('product/' + product.productID +
            '/delivery_range/' + districtID);
            promise.then(function (result) {
                if (result.data.success &&
                    result.data.deliveryRange.deliveryAvailable) {
                    product.unreachable = false;
                } else {
                    vm.canSubmit = false;
                }
            });
        }

        function refreshAddress(event, address, products) {
            vm.address = address;
            $scope.$digest();
            updateProduct(products);
        }

        function refreshInvoice(event, title) {
            vm.invoiceTitle = title;
        }

        function submitOrder() {
            var sales = [];
            angular.forEach(vm.products, function (product) {
                sales.push({
                    sale: product.id,
                    quantity: product.quantity
                });
            });
            var promise = httpService.post('order', {
                address: vm.address.id,
                sales: sales,
                direct_buy: $stateParams.isDirectBuy,
                invoice_title: vm.invoiceTitle
            });
            promise.then(function (result) {
                cartService.refreshCart();
                routerStateService.go('pay', {transId: result.data.transId});
            });
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.order')
        .controller('CustomInvoiceController', CustomInvoiceController);

    CustomInvoiceController.$inject = ['$timeout', '$rootScope', '$ionicHistory'];

    function CustomInvoiceController($timeout, $rootScope, $ionicHistory) {


        var vm = this;
        vm.invoiceTitle = null;
        vm.updateTitle = updateTitle;

        ////////////////

        function updateTitle() {
            $ionicHistory.goBack();
            $timeout(function () {
                $rootScope.$broadcast('checkout:refreshInvoice', vm.invoiceTitle);
            }, 10);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.order')
        .controller('PaymentSuccessController', PaymentSuccessController);

    PaymentSuccessController.$inject = ['$ionicHistory', 'routerStateService'];

    function PaymentSuccessController($ionicHistory, routerStateService) {
        var vm = this;
        vm.goBack = goBack;
        vm.goDashboard = goDashboard;

        ////////////////

        function goBack() {
            $ionicHistory.goBack(-2);
        }

        function goDashboard() {
            routerStateService.go('tab.dashboard');
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.order')
        .controller('PaymentController', PaymentController);

    PaymentController.$inject = ['$stateParams', '$ionicHistory', 'httpService', 'routerStateService', 'alertService'];

    function PaymentController($stateParams, $ionicHistory, httpService, routerStateService, alertService) {
        var vm = this;
        var type = 'alipay';
        vm.transId = $stateParams.transId;
        vm.goBack = goBack;
        vm.pay = pay;
        vm.selectType = selectType;

        ////////////////

        function goBack() {
            $ionicHistory.goBack(-2);
        }

        function pay() {
            if (type === 'alipay') {
                var promise = httpService.post('payment/' + vm.transId + '/alipay_app');
                promise.then(function (result) {
                    if (result.data.success) {
                        vm.transId = result.data.transId;
                        if (typeof cordova !== 'undefined') {
                            HBAlipay.pay(result.data.paymentQuery, 'arkenstone', onSuccess, onFailed);
                        } else {
                            alertService.autoShow('当前设备无法使用支付宝付款');
                        }
                    } else {
                        alertService.autoShow(result.data.errorResponse.errMsg);
                    }
                });
            } else if (type === 'wechat') {
                var promise = httpService.post('payment/' + vm.transId + '/wxpay_app');
                promise.then(function (result) {
                    if (result.data.success) {
                        vm.transId = result.data.transId;
                        if (typeof cordova !== 'undefined') {
                            HBWechat.pay(result.data.prepayReq, onSuccess, onFailed);
                        } else {
                            alertService.autoShow('当前设备无法使用微信付款');
                        }
                    } else {
                        alertService.autoShow(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function selectType(_type) {
            type = _type;
        }

        function onFailed(message) {
            alertService.autoShow(message || '支付失败');
        }

        function onSuccess(message) {
            alertService.autoShow(message || '支付成功！');
            routerStateService.go('pay-success');
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('alertService', alertService);

    alertService.$inject = ['$templateRequest', '$timeout'];

    function alertService($templateRequest, $timeout) {
        var tpl = '';
        var height = $(window).height();
        var loader = $templateRequest('templates/partials/alert-box.html', true);
        loader.then(function(respones) {
            tpl = respones;
        });

        return {
            show: function(msg, top) {
                var t = top || height/4-30;
                this.$tpl = $(tpl);
                this.$tpl.find('.info-box').css('margin-top', t);
                this.$tpl.find('.text').text(msg);
                this.$tpl.appendTo(document.body);
            },
            autoShow: function(msg, top, ms) {
                var timeout_ms = ms || 2000;
                var self = this;
                this.show(msg, top);
                $timeout(function() {
                    self.close(200);
                }, timeout_ms);
            },
            close: function(seconds) {
                var self = this;
                if(seconds) {
                    this.$tpl.animate({
                        opacity: 0
                    }, seconds, function() {
                        self.$tpl.remove();
                    });
                } else {
                    this.$tpl.remove();
                }
            }
        };
    }
})();

(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('authService', authService);

    authService.$inject = ['$rootScope', 'localStorageService'];

    function authService($rootScope, localStorageService) {
        var userName;
        var service = {
            getAuthenticatedUser: getAuthenticatedUser,
            isAuthorized: isAuthorized,
            signIn: signIn,
            signOut: signOut,
            updateUser: updateUser
        };
        return service;

        ////////////////

        function getAuthenticatedUser() {
            return localStorageService.getObject('ArkenstoneUser');
        }

        function isAuthorized() {
            var user = getAuthenticatedUser();
            if (user) {
                userName = user.userName;
            }
            return user !== null;
        }

        function signIn(user) {
            userName = user.userName;
            localStorageService.setObject('ArkenstoneUser', user);
            $rootScope.$broadcast('cartService:refreshCart');
        }

        function signOut() {
            localStorageService.setObject('ArkenstoneUser', null);
            $rootScope.$broadcast('cartService:refreshCart');
            $rootScope.$broadcast('auth:signout');
        }

        function updateUser(user) {
            localStorageService.setObject('ArkenstoneUser', user);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('cartService', cartService);

    cartService.$inject = ['$rootScope', 'httpService'];

    function cartService($rootScope, httpService) {
        var productsData = null;
        var service = {
            addToCart: addToCart,
            bulkUpdateCart: bulkUpdateCart,
            flyToCart: flyToCart,
            getTotalQuantity: getTotalQuantity,
            getProductsInCart: getProductsInCart,
            updateCart: updateCart,
            removeFromCart: removeFromCart,
            bulkRemoveFromCart: bulkRemoveFromCart,
            refreshCart: refreshCart
        };
        return service;

        ////////////////

        function refreshCart(cbFunc) {
            var promise = httpService.get('cart');
            promise.then(function (result) {
                productsData = result.data;
                angular.forEach(productsData.sales, function (product) {
                    if (product.statsInfo.completed) {
                        product.checked = false;
                    }
                });
                $rootScope.$broadcast('cartService:refreshCountInCart');
                if (cbFunc && typeof cbFunc === 'function') {
                    cbFunc();
                }
            });
        }

        function getProductsInCart() {
            return productsData.sales;
        }

        function getTotalQuantity() {
            return productsData.totalQuantity;
        }

        function addToCart(id, quantity, cbFunc) {
            var promise = httpService.post('cart/add', {
                sale: id,
                action: 'add',
                quantity: quantity
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function updateCart(id, quantity, checked, cbFunc) {
            var promise = httpService.post('cart/add', {
                sale: id,
                action: 'update',
                quantity: quantity,
                checked: checked
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function bulkUpdateCart(products, cbFunc) {
            var promise = httpService.post('cart/bulk_update', {
                sales: products
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function removeFromCart(id, cbFunc) {
            var promise = httpService.post('cart/del', {
                sale: id
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function bulkRemoveFromCart(ids, cbFunc) {
            var promise = httpService.post('cart/bulk_delete', {
                sales: ids
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function flyToCart(productImgUrl) {
            event.cancelBubble = true;
            var cartTab = $('a.cart-tab');
            var cartTabPosition = cartTab[0].getBoundingClientRect();
            var startPosition = {
                x: 0,
                y: 0
            };
            var endPosition = {
                x: cartTabPosition.left - event.x + cartTab.width() / 2,
                y: cartTabPosition.top - event.y
            };
            var controlPosition = {
                x: endPosition.x,
                y: 0
            };
            var fakeProduct = cloneFlyingProduct(event.x, event.y, productImgUrl);
            fly(startPosition, endPosition, controlPosition, 0, 0, fakeProduct);
        }

        function cloneFlyingProduct(x, y, imgURL) {
            var fakeProduct = $('<div class="product-shadow"><img src=' + imgURL + '></div>').css({
                'left': x + 'px',
                'top': y + 'px'
            });
            $(document.body).append(fakeProduct);
            return fakeProduct;
        }

        function fly(sp, ep, cp, i, factor, fakeProduct) {
            if (i <= 100) {
                var step = twoBessel(sp, ep, cp, i);
                fakeProduct.css('transform', 'translate(' + step.x + 'px, ' + step.y + 'px)');
                i += factor += 0.15;
                setTimeout(function () {
                    fly(sp, ep, cp, i, factor, fakeProduct);
                }, 15);
            } else {
                fakeProduct.remove();
            }
        }

        function twoBessel(sp, ep, cp, step) {
            var t = step / 100;
            var a = t * t;
            var b = 2 * t * (1 - t);
            var c = Math.pow(1 - t, 2);
            return {
                x: c * sp.x + b * cp.x + a * ep.x,
                y: c * sp.y + b * cp.y + a * ep.y
            };
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('currentCityService', currentCityService);

    currentCityService.$inject = ['$q', '$timeout', '$rootScope', 'localStorageService', 'httpService'];

    function currentCityService ($q, $timeout, $rootScope, localStorageService, httpService) {
        var city_key = 'current_city';
        var gaodeScript, provinces, cities = {};
        var position_option = {
            enableHighAccuracy: true,
            maximumAge:  3600000,  // 1 hour
            timeout: 10000
        };
        var service = {
            fetchCities: fetchCities,
            setCurrentCity: setCurrentCity,
            setCurrentCityByName: setCurrentCityByName,
            getCurrentCity: getCurrentCity,
            clearCurrentCity: clearCurrentCity,
            locateByGPS: locateByGPS
        };

        $rootScope.$on('auth:signout', clearCurrentCity);

        init();

        return service;

        ////////////////
        function init() {
            fetchCities().then(function (resp) {
                resp.data.provinceListWithCities.forEach(function (province) {
                    province.cities.forEach(function (city) {
                        cities[city.name] = city;
                    });
                });
            });
        }

        function fetchCities() {
            if (!provinces) {
                provinces = httpService.get('province_list_with_cities');
            }
            return provinces
        }

        function getCurrentCity() {
            return localStorageService.getObject(city_key);
        }

        function setCurrentCity(city) {
            localStorageService.setObject(city_key, city);
            $rootScope.$broadcast('currentCity:changed');
        }

        function setCurrentCityByName(cityName) {
            if (cityName in cities) {
                setCurrentCity(cities[cityName]);
            } else {
                debug('city not found by name: ' + cityName);
            }
        }

        function clearCurrentCity() {
            setCurrentCity(null);
        }

        function loadGaodeScript () {
            // asynchronously load gaode script
            if (!gaodeScript) {
                var deferred = $q.defer();
                var script = document.createElement('script');
                script.src = window.ArkenstoneAppSettings.gaodeScriptUrl +
                    '&key=' + window.ArkenstoneAppSettings.gaodeApiKey;
                document.body.appendChild(script);

                script.onload = script.onreadystatechange = function (e) {
                    $timeout(function () {
                        deferred.resolve(e);
                    });
                };

                script.onerror = function (e) {
                    $timeout(function () {
                        deferred.reject(e);
                    });
                };

                gaodeScript = deferred.promise;
            }
            return gaodeScript;
        }

        function locateByGPS() {
            var deferred = $q.defer();
            document.addEventListener("deviceready", function () {
                if ('geolocation' in navigator) {
                    navigator.geolocation.getCurrentPosition(function (position) {
                        // now that we get GPS coordinates, load gaode stuff
                        loadGaodeScript().then(function () {
                            AMap.service(['AMap.Geocoder'], function () {
                                var geocoder = new AMap.Geocoder({
                                    extensions: 'all'
                                });
                                var point = new AMap.LngLat(position.coords.longitude, position.coords.latitude);
                                geocoder.getAddress(point, function (status, result) {
                                    if (status === 'complete' && result.info == 'OK') {
                                        var addr = result.regeocode.addressComponent;
                                        var city = Boolean(addr.city) ? addr.city: addr.province;
                                        // remove trailing '市'
                                        if (city.match(/市$/)) {  // safari doesn't support String.prototype.endsWith
                                            city = city.slice(0, city.length - 1);
                                        }
                                        deferred.resolve(city);
                                    } else {
                                        deferred.reject('Gaode cannot geocode coordinate ' + point + ': ' + status);
                                    }
                                });
                            });
                        });
                    }, function (error) {
                        deferred.reject(error.message);
                    }, position_option);
                } else {
                    deferred.reject('navigator.geolocation not available');
                }
            }, false);

            deferred.promise.catch(function (reason) {
                debug('Geolocation error: ' + reason);
                return reason;
            });
            return deferred.promise;
        }

        function debug (message) {
            if (window.ArkenstoneAppSettings.debug) {
                console.log(message);
            }
        }
    }

})();

(function() {
    'use strict';

    angular
        .module('app.service')
        .factory('httpInterceptor', httpInterceptor);

    httpInterceptor.$inject = ['$rootScope', '$q', 'authService'];

    function httpInterceptor($rootScope, $q, authService) {
        var service = {
            request: requestHandler,
            requestError: requestErrorHandler,
            response: responseHandler,
            responseError: responseErrorHandler

        };
        return service;

        ////////////////

        function requestHandler(config) {
            if (config.responseType === 'json') {
                $rootScope.$broadcast('loading:show');
            }
            return config;
        }

        function requestErrorHandler(rejection) {
            return $q.reject(rejection);
        }

        function responseHandler(response) {
            $rootScope.$broadcast('loading:hide');
            if (response.data.error === undefined) {
                return response;
            } else {
                $rootScope.$broadcast('error:show', response.data.error);
                return $q.reject(response);
            }
        }

        function responseErrorHandler(rejection) {
            $rootScope.$broadcast('loading:hide');
            // TODO: Log error
            var message = '服务器错误，请稍后重试';
            var nextState = null;
            if (rejection.status === 400) {
                message = '错误请求';
            } else if (rejection.status === 401) {
                authService.signOut();
                message = '会话已过期，请重新登录';
                nextState = 'login';
            } else if (rejection.status === 404) {
                message = '所访问资源不存在';
            } else if (rejection.data && rejection.data.error) {
                message = rejection.data.error.message;
            }
            $rootScope.$broadcast('error:show', {
                code: rejection.status,
                message: message,
                nextState: nextState
            });
            return $q.reject(rejection);
        }
    }
})();

(function() {
    'use strict';

    angular
        .module('app.service')
        .factory('httpService', httpService);

    httpService.$inject = ['$http'];

    function httpService($http) {
        var service = {
            get: get,
            post: post
        };
        return service;

        ////////////////

        function get(url, params) {
            params = params || {};
            params.scope = 'app';
            return $http({
                url: window.ArkenstoneAppSettings.baseApiUrl + url,
                method: 'GET',
                params: params,
                responseType: 'json'
            });
        }

        function post(url, data, config) {
            data = data || {};
            data.scope = 'app';
            config = config || {};
            config.responseType = 'json';
            return $http.post(
                window.ArkenstoneAppSettings.baseApiUrl + url,
                data, config);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('localStorageService', localStorageService);

    localStorageService.$inject = ['$window'];

    function localStorageService($window) {
        var service = {
            getObject: getObject,
            getValue: getValue,
            setObject: setObject,
            setValue: setValue
        };
        return service;

        ////////////////

        function getObject(key) {
            return JSON.parse($window.localStorage[key] || 'null');
        }

        function getValue(key, defaultValue) {
            return $window.localStorage[key] || defaultValue;
        }

        function setObject(key, objData) {
            $window.localStorage[key] = JSON.stringify(objData);
        }

        function setValue(key, value) {
            $window.localStorage[key] = value;
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('modalManagerService', modalManagerService);

    modalManagerService.$inject = ['$ionicModal'];


    function modalManagerService($ionicModal) {
        var modalList = {
            about: {
                templateUrl: 'templates/partials/about.modal.html'
            },
            terms: {
                templateUrl: 'templates/partials/terms.modal.html'
            },
            orderReview: {
                templateUrl: 'templates/partials/order-review.modal.html'
            },
            productDeliveryArea: {
                templateUrl: 'templates/partials/delivery-list.modal.html',
                animation: 'slide-in-left',
                onShown: function (modal) {
                    modal.scope.$broadcast('deliveryArea:show', modal);
                }
            },
            userDeliveryArea: {
                templateUrl: 'templates/partials/delivery-list.modal.html',
                onShown: function (modal) {
                    modal.scope.$broadcast('deliveryArea:show', modal);
                }
            }
        };
        var service = {
            addModal: addModal,
            getModal: getModal,
            showModal: showModal,
            closeModalByName: closeModalByName,
            closeModal: closeModal
        };
        return service;

        ////////////////

        function getModalName(name) {
            return name + 'Modal';
        }

        function addModal(name, scope, options, show_immediately) {
            if (!(name && name.trim())) {
                return;
            }
            var modal = getModal(name, scope);
            if (modal) {
                if (show_immediately) {
                    modal.show();
                }
                return;
            }
            var modalOptions = angular.extend({}, modalList[name] || {}, options || {});
            $ionicModal.fromTemplateUrl(modalOptions.templateUrl, {
                scope: scope,
                animation: modalOptions.animation || 'slide-in-up'
            }).then(function (modal) {
                var modalName = getModalName(name);
                scope[modalName] = modal;
                modal.$el.addClass(modalName.replace(/([A-Z])/g, '-$1').toLowerCase());
                modal.$el.css('background', 'none');
                if (show_immediately) {
                    modal.show();
                }
            });

            scope.$on('$destroy', function () {
                var modal = getModal(name, scope);
                if (modal) {
                    modal.remove();
                }
            });

            scope.$on('modal.shown', function(event, modal) {
                console.info(getModalName(name) + ' is shown!');

                if (modalOptions.onShown) {
                    modalOptions.onShown(modal);
                }
            });

            scope.$on('modal.hidden', function(event, modal) {
                console.info(getModalName(name) + ' is hidden!');

                if (modalOptions.onHidden) {
                    modalOptions.onHidden(modal);
                }
            });

            scope.$on('modal.removed', function() {
                console.info(getModalName(name) + ' is removed!');
            });

        }

        function getModal(name, scope) {
            return scope[getModalName(name)];
        }

        function showModal(name, scope, options) {
            addModal(name, scope, options, true);
        }

        function closeModalByName(name, scope, onClose) {
            closeModal(getModal(name, scope), onClose);
        }

        function closeModal(modal, onClose) {
            if (modal) {
                if (onClose) {
                    onClose(modal);
                }
                modal.hide();
            }
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('productSearchService', productSearchService);

    productSearchService.$inject = ['appConf', 'httpService', 'currentCityService'];

    function productSearchService(appConf, httpService, currentCityService) {
        var params;
        resetParams();

        var service = {
            getParam: getParam,
            resetParams: resetParams,
            search: search,
            setParam: setParam
        };
        return service;

        function search(page, limit) {
            page = page || 0;
            limit = limit || appConf.maxItemsDisplayedInList;
            var data = {
                page: page,
                length: limit,
                keyword: params.keyword,
                ordered: params.orderBy[0],
                reverse: params.orderBy[1]
            };
            var city = currentCityService.getCurrentCity();
            if (city) {
                data.current_location = city.id;
            }
            if (params.productType) {
                data.product_type = params.productType;
            }
            return httpService.get('product_list', data);
        }

        function getParam(name) {
            return params[name];
        }

        function setParam(name, value) {
            params[name] = value;
        }

        function resetParams() {
            params = {
                productType: 0,
                keyword: '',
                orderBy: ['newest', true],
                province: 0
            };
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('routerStateService', routerStateService);

    routerStateService.$inject = ['$state'];

    function routerStateService($state) {
        var service = {
            go: go,
            getCurrentName: getCurrentName,
            reload: reload
        };
        return service;

        ////////////////

        function go(to, params, options) {
            if (to.indexOf('tab.') !== -1) {
                $state.go(to, params, options);
            } else {
                var currentState = getCurrentName(),
                    currentTab = null;
                if (currentState.indexOf('tab.') === -1) {
                    currentTab = currentState.split('-')[0];
                } else {
                    currentTab = currentState.split('.')[1];
                }
                $state.go(currentTab + '-' + to, params, options);
            }
        }

        function getCurrentName() {
            return $state.$current.name;
        }

        function reload() {
            $state.reload;
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('AboutController', AboutController);

    AboutController.$inject = ['$scope', '$sce', 'modalManagerService', 'httpService'];

    function AboutController($scope, $sce, modalManagerService, httpService) {
        var vm = this;
        vm.closeModal = closeModal;
        vm.targetURL = '';

        init();

        ////////////////

        function init() {
            var promise = httpService.get('about_us_url');
            promise.then(function (result) {
                vm.targetURL = $sce.trustAsResourceUrl(result.data.url);
            });
        }

        function closeModal() {
            modalManagerService.closeModalByName('about', $scope);
        }
    }

})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('AccountController', AccountController);

    AccountController.$inject = ['httpService', 'authService'];

    function AccountController(httpService, authService) {
        var vm = this;
        vm.signOut = signOut;
        vm.user = null;

        fetchUser();

        ////////////////

        function fetchUser() {
            vm.user = authService.getAuthenticatedUser();
        }

        function signOut() {
            var promise = httpService.get('signout');
            promise.then(function (result) {
                authService.signOut();
                fetchUser();
            });
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('BindStrategyController', BindStrategyController);

    BindStrategyController.$inject = ['$scope', 'httpService', 'alertService',
        '$stateParams', '$ionicHistory', 'authService'];

    function BindStrategyController($scope, httpService, alertService,
                                    $stateParams, $ionicHistory, authService) {
        var vm = this;
        var regExpPhone = /^[0-9]{11}$/;
        var regExpEmail = /^.+@[^.].*.[a-z]{2,10}$/;

        vm.code = '';
        vm.btnText = '';
        vm.title = '';
        vm.type = '';
        vm.typeText = '';
        vm.typeValue = '';
        vm.isBind = true;
        vm.fieldCode = '获取验证码';
        vm.phone_error_text = '手机号格式不正确!';
        vm.email_error_text = '邮箱格式不正确!';
        vm.code_error_text = '验证码不能为空!';
        vm.flag = false;
        vm.getCode = getCode;
        vm.bind = bind;

        init();

        function init() {
            if ($stateParams.isBind) {
                if ($stateParams.type === 'phone') {
                    vm.title = '绑定手机号';
                    vm.typeText = '手机号';
                } else if ($stateParams.type === 'email') {
                    vm.title = '绑定邮箱';
                    vm.typeText = '邮箱';
                }
                vm.btnText = '绑定';
            } else {
                if ($stateParams.type === 'phone') {
                    vm.title = '解绑手机号';
                    vm.typeText = '手机号';
                } else if ($stateParams.type === 'email') {
                    vm.title = '解绑邮箱';
                    vm.typeText = '邮箱';
                }
                vm.typeValue = $stateParams.typeValue;
                vm.btnText = '解绑';
            }
            vm.isBind = $stateParams.isBind;
            vm.type = $stateParams.type;
        }

        function bind() {
            var url;
            if (validate()) {
                if ($stateParams.type === 'phone') {
                    url = vm.isBind ? 'user/bind/mobile' : 'user/unbind/mobile';
                } else if ($stateParams.type === 'email') {
                    url = vm.isBind ? 'user/bind/email' : 'user/unbind/email';
                }
                httpService.post(url, {
                    address: vm.typeValue,
                    verify_code: vm.code
                }).then(function (result) {
                    if (result.data.success) {
                        updateUser();
                        $ionicHistory.goBack();
                    } else if (result.data.errorResponse) {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function updateUser() {
            var user = authService.getAuthenticatedUser();
            if ($stateParams.isBind) {
                if ($stateParams.type === 'phone') {
                    user.mobile = vm.typeValue;
                } else if ($stateParams.type === 'email') {
                    user.email = vm.typeValue;
                }
            } else {
                if ($stateParams.type === 'phone') {
                    user.mobile = '';
                } else if ($stateParams.type === 'email') {
                    user.email = '';
                }
            }
            authService.updateUser(user);
        }

        function validate() {
            if (vm.isBind) {
                if ($stateParams.type === 'phone') {
                    if (!regExpPhone.test(vm.typeValue)) {
                        showPrompt(vm.phone_error_text);
                        return false;
                    }
                } else if ($stateParams.type === 'email') {
                    if (!regExpEmail.test(vm.typeValue)) {
                        showPrompt(vm.email_error_text);
                        return false;
                    }
                }
            } else {
                if ($.trim(vm.code) === '') {
                    showPrompt(vm.code_error_text);
                    return false;
                }
            }

            return true;
        }

        function getCode() {
            var url, regExp, info, address, need_user, parms;
            if ($stateParams.type === 'phone') {
                url = 'verification/mobile';
                address = vm.typeValue;
                need_user = true;
                if (vm.isBind) {
                    need_user = false;
                    regExp = regExpPhone;
                    info = vm.phone_error_text;
                }
            } else if ($stateParams.type === 'email') {
                url = 'verification/email';
                address = vm.typeValue;
                need_user = true;
                if (vm.isBind) {
                    need_user = false;
                    regExp = regExpEmail;
                    info = vm.email_error_text;
                }
            }
            if ( (regExp && regExp.test(address)) || !vm.isBind) {
                if (need_user) {
                    parms = {
                        address: address,
                        need_user: need_user
                    };
                } else {
                    parms = {
                        address: address
                    };
                }
                httpService.post(url, parms).then(function (result) {
                    if (result.data.success) {
                        var count = result.data.resendInterval;
                        vm.fieldCode = count + '秒后' + vm.fieldCode;
                        vm.flag = !vm.flag;
                        var timer = setInterval(function () {
                            if (count < 1) {
                                vm.fieldCode = '获取验证码';
                                vm.flag = !vm.flag;
                                $scope.$digest();
                                clearInterval(timer);
                                return;
                            }
                            count -= 1;
                            vm.fieldCode = count + '秒后获取验证码';
                            $scope.$digest();
                        }, 1000);
                    } else if (result.data.errorResponse) {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            } else {
                showPrompt(info);
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('ChangePasswordController', ChangePasswordController);

    ChangePasswordController.$inject = ['alertService', '$stateParams', 'httpService', '$ionicHistory'];

    function ChangePasswordController(alertService, $stateParams, httpService, $ionicHistory) {
        var vm = this;
        vm.oldPassword = '';
        vm.newPassword = '';
        vm.confirmPassword = '';
        vm.onChangePassword = onChangePassword;
        vm.onResetPassword = onResetPassword;

        ////////////////

        function onChangePassword() {
            if (validate(true)) {
                httpService.post('user/reset_password', {
                    'old_password': vm.oldPassword,
                    'new_password': vm.newPassword,
                    're_password': vm.confirmPassword
                }).then(function (result) {
                    if (result.data.success) {
                        $ionicHistory.goBack(-2);
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function onResetPassword() {
            if (validate(false)) {
                httpService.post('user/reset_password', {
                    token: $stateParams.token,
                    new_password: vm.newPassword,
                    re_password: vm.confirmPassword
                }).then(function (result) {
                    if (result.data.success) {
                        $ionicHistory.goBack(-2);
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function validate(need_old_pw) {
            var is_ok = false;
            if (need_old_pw && vm.oldPassword.length <= 0) {
                showPrompt('请输入当前密码')
            } else if (vm.newPassword.length <= 0) {
                showPrompt('请输入新密码')
            } else if (!/^[^\n\r]{6,20}$/.test(vm.newPassword)) {
                showPrompt('新密码长度不符合要求')
            } else if (vm.confirmPassword.length <= 0) {
                showPrompt('请再次输入新密码')
            } else if (vm.newPassword !== vm.confirmPassword) {
                showPrompt('两次输入的密码不一样')
            } else {
                is_ok = true;
            }
            return is_ok;
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();


(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('ChargeRedemptionConController', ChargeRedemptionConController);

    ChargeRedemptionConController.$inject = ['alertService', 'httpService', 'routerStateService'];

    function ChargeRedemptionConController(alertService, httpService, routerStateService) {
        var vm = this;
        vm.redemptionCode = null;
        vm.chargeRedemption = chargeRedemption;

        function chargeRedemption() {
            var promise = httpService.post('charge_redemption_code', {code: vm.redemptionCode});
            promise.then(function(response) {
                if (response.data.success) {
                    routerStateService.go('user-credits');
                } else {
                    alertService.autoShow(response.data.errorResponse.errMsg);
                }
            });
        }
    }

})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('GeoLocationController', GeoLocationController);

    GeoLocationController.$inject = ['currentCityService', '$ionicPopup'];

    function GeoLocationController(currentCityService, $ionicPopup) {
        init();

        ///////////////////////////
        function init () {
            var city = currentCityService.getCurrentCity();
            // if city already set, don't touch GPS and gaode script at all
            if (!city) {
                // try to locate user's city by GPS
                currentCityService.locateByGPS()
                    .then(function (cityName) {
                        $ionicPopup.confirm({
                            title: '是否切换城市至' + cityName + '市',
                            okText: '确定',
                            cancelText: '取消'
                        }).then(function (confirmed) {
                            if (confirmed) {
                                currentCityService.setCurrentCityByName(cityName);
                            }
                        });
                    })

            }
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('LoginController', LoginController);

    LoginController.$inject = ['alertService', '$timeout', '$ionicHistory', 'httpService', 'authService', 'routerStateService'];

    function LoginController(alertService, $timeout, $ionicHistory, httpService, authService, routerStateService) {
        var vm = this;
        vm.onLogin = onLogin;
        vm.onRegister = onRegister;
        vm.onForgotPassword = onForgotPassword;
        vm.password = '';
        vm.userName = '';

        ////////////////

        function onLogin() {
            var lenUserName = vm.userName.replace(/(^s*)|(s*$)/g, '').length;
            var lenPassword = vm.password.replace(/(^s*)|(s*$)/g, '').length;

            if (lenUserName === 0 && lenPassword === 0) {
                showPrompt('请输入用户名和密码');
            } else if (lenUserName === 0) {
                showPrompt('请输入用户名');
            } else if (lenPassword === 0) {
                showPrompt('请输入密码');
            } else {
                var reqData = {
                    username: vm.userName,
                    password: vm.password,
                    remember: true
                };
                var promise = httpService.post('signin', reqData);
                promise.then(function (result) {
                    if (result.data.success) {
                        authService.signIn(result.data.userInfo);
                        $ionicHistory.goBack();
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }

        function onRegister() {
            routerStateService.go('register');
        }

        function onForgotPassword() {
            routerStateService.go('retrieve-password');
        }
    }

})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('OrderInfoController', OrderInfoController);

    OrderInfoController.$inject = ['$scope', 'httpService', 'routerStateService',
        '$stateParams', 'appConst'];

    function OrderInfoController($scope, httpService, routerStateService, $stateParams, appConst) {
        var vm = this;
        vm.order = null;
        vm.orderRefund = orderRefund;
        $scope.appConst = appConst;

        readInfo();
        ////////////////

        function readInfo() {
            var promise = httpService.get('order/' + $stateParams.orderId);
            promise.then(function (result) {
                if (result.data.success && result.data.order) {
                    vm.order = result.data.order;
                }
            });
        }

        function orderRefund() {
            routerStateService.go('order-refund', {orderId: $stateParams.orderId});
        }

    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('OrderListController', OrderListController);

    OrderListController.$inject = ['$scope', 'httpService', 'routerStateService', 'appConst', 'modalManagerService'];

    function OrderListController($scope, httpService, routerStateService, appConst, modalManagerService) {
        var vm = this;
        vm.getOrderByCondition = getOrderByCondition;
        vm.hasMore = true;
        vm.onReview = onReview;
        vm.onPaying = onPaying;
        vm.onOrderInfo = onOrderInfo;
        vm.orderList = [];
        vm.page = 0;
        $scope.appConst = appConst;

        ////////////////

        function onPaying(order) {
            routerStateService.go('pay', {transId: order.trans_id});
        }

        function onReview(order) {
            $scope.order = order;
            $scope.reload = false;
            modalManagerService.showModal('orderReview', $scope, {
                onHidden: function (modal) {
                    if (modal.scope.reload) {
                        order.reviewed = true;
                        routerStateService.reload();
                    }
                }
            });
        }

        function onOrderInfo(order) {
            routerStateService.go('order-info', {orderId: order.id});
        }

        function getOrderByCondition() {
            if (!vm.hasMore) {
                return;
            }

            var orderList = httpService.get('order_list', {page: vm.page});
            orderList.then(function (result) {
                vm.orderList = vm.orderList.concat(result.data.orderList);
                if (result.data.orderList.length === 0) {
                    vm.hasMore = false;
                } else {
                    vm.page++;
                }
            }).catch(function () {
                vm.hasMore = false;
            }).finally(function () {
                $scope.$broadcast('scroll.infiniteScrollComplete');
            });
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('OrderRefundController', OrderRefundController);

    OrderRefundController.$inject = ['httpService', 'routerStateService', '$stateParams',
        '$ionicPopup', '$ionicViewSwitcher', 'alertService'];

    function OrderRefundController(httpService, routerStateService, $stateParams, $ionicPopup, $ionicViewSwitcher, alertService) {
        var vm = this;
        vm.reason = '';
        vm.voucher1 = null;
        vm.voucher2 = null;
        vm.voucher3 = null;
        vm.uploadImg = uploadImg;
        vm.removeImg = removeImg;
        vm.sendOrderRefundReq = sendOrderRefundReq;

        function uploadImg() {
            $('input[type=file]').each(function (index) {
                if ($(this).val() === '') {
                    $(this).click();
                    return false;
                }
                if (index === 2) {
                    alertService.autoShow('最多上传三张图片');
                }
            });
        }

        function removeImg(name) {
            $ionicPopup.confirm({
                title: '确认删除该图片吗',
                okText: '确定',
                okType: 'button-calm',
                cancelText: '取消'
            }).then(function (result) {
                if (result) {
                    var input_ctl = $('#' + name);
                    input_ctl.val('');
                    $('img#' + name + '-img').attr('src', '');
                    //$('img#' + name + '-img').css('display', 'none');
                    $('img#' + name + '-img').hide();
                }
            });
        }

        function sendOrderRefundReq() {
            if (vm.reason === '') {
                alertService.autoShow('请填写退款申请原因');
                return;
            }
            var data = new FormData();
            data.append('reason', vm.reason);
            if (vm.voucher1) {
                data.append('voucher-1', vm.voucher1);
            }
            if (vm.voucher2) {
                data.append('voucher-2', vm.voucher2);
            }
            if (vm.voucher3) {
                data.append('voucher-3', vm.voucher3);
            }

            var promise = httpService.post('order/' + $stateParams.orderId + '/refund', data, {
                transformRequest: angular.identity,
                headers: {'Content-Type': undefined}
            });
            promise.then(function (result) {
                if (result.data.success) {
                    $ionicViewSwitcher.nextDirection('back');
                    routerStateService.go('order-info', {orderId: $stateParams.orderId});
                }
            });
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('OrderReviewController', OrderReviewController);

    OrderReviewController.$inject = ['$scope', 'httpService', 'modalManagerService'];

    function OrderReviewController($scope, httpService, modalManagerService) {
        var vm = this;
        vm.rating = 5;
        vm.review = '';
        vm.closeModal = closeModal;
        vm.onSend = onSend;

        function onSend() {
            var order_id = $scope.order.id;
            var promise = httpService.post('order/' + order_id + '/review', {
                rating: vm.rating,
                review: vm.review
            });
            promise.then(function (result) {
                if (result.data.success) {
                    modalManagerService.closeModalByName('orderReview', $scope, function (modal) {
                        modal.scope.reload = true;
                    });
                }
            });
        }

        function closeModal() {
            modalManagerService.closeModalByName('orderReview', $scope);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('RegisterController', RegisterController);

    RegisterController.$inject = ['alertService', '$scope', 'httpService', '$ionicHistory', 'authService', 'modalManagerService', 'routerStateService'];

    function RegisterController(alertService, $scope, httpService, $ionicHistory, authService, modalManagerService, routerStateService) {
        var vm = this;
        vm.username = '';
        vm.password = '';
        vm.repeatPassword = '';
        vm.phone = '';
        vm.code = '';
        vm.second = '';
        vm.disable = false;
        vm.flag = false;

        vm.back = back;
        vm.register = register;
        vm.onLogin = onLogin;
        vm.showTermsModal = showTermsModal;
        vm.getCode = getCode;
        vm.finish = finish;

        ////////////////

        function back() {
            if (vm.flag) {
                vm.flag = false;
            } else {
                $ionicHistory.goBack();
            }
        }

        function register() {
            var flag = validate();
            if (flag) {
                var promise = httpService.post('signup', {
                    username: vm.username,
                    password: vm.password,
                    repassword: vm.repeatPassword,
                    pre_signup: true
                });
                promise.then(function (result) {
                    if (result.data.success) {
                        vm.flag = true;
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }

        function validate() {
            if (vm.username.replace(/^\s+|\s+$/g, '') == '') {
                showPrompt('用户名不能为空!');
                return false;
            } else if (!/^[0-9|a-z|A-Z|_]{4,20}$/.test(vm.username)) {
                showPrompt('请输入正确的用户名!');
                return false;
            }

            if (vm.password == '') {
                showPrompt('密码不能为空!');
                return false;
            } else if (!/^[^\n\r]{6,20}$/.test(vm.password)) {
                showPrompt('密码长度不符合要求!');
                return false;
            }

            if (vm.repeatPassword == '') {
                showPrompt('确认密码不能为空!');
                return false;
            } else if (vm.password != vm.repeatPassword) {
                showPrompt('两次输入的密码不一致!');
                return false;
            }

            return true;
        }

        function validatePhone() {
            if (!/^[0-9]{11}$/.test(vm.phone)) {
                showPrompt('请输入正确的手机号!');
                return false;
            }
            return true;
        }

        function validateCode() {
            if ($.trim(vm.code) == '') {
                showPrompt('验证码不能为空!');
                return false;
            }
            return true;
        }

        function getCode() {
            if (validatePhone()) {
                vm.disable = true;
                getServerCode(function (soconds) {
                    var s = soconds;

                    vm.second = '（' + s + '）';
                    var timer = setInterval(function () {
                        s--;
                        if (s < 0) {
                            clearInterval(timer);
                            vm.second = '';
                            vm.disable = false;
                        } else {
                            vm.second = '（' + s + '）';
                        }
                        $scope.$digest();

                    }, 1000);
                });
            }
        }

        function getServerCode(callback) {
            httpService.post('verification/mobile', {
                address: vm.phone
            }).then(function (result) {
                if (result.data.success) {
                    callback(result.data.resendInterval);
                } else {
                    vm.disable = false;
                    showPrompt(result.data.errorResponse.errMsg);
                }
            });
        }

        function finish() {
            if (validatePhone() && validateCode()) {
                httpService.post('register', {
                    username: vm.username,
                    password: vm.password,
                    repassword: vm.repeatPassword,
                    media: 'mobile',
                    address: vm.phone,
                    keycode: vm.code
                }).then(function (result) {
                    if (result.data.success) {
                        authService.signIn(result.data.userInfo);
                        $ionicHistory.goBack();
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function showTermsModal() {
            modalManagerService.showModal('terms', $scope);
        }

        function onLogin() {
            routerStateService.go('login');
        }
    }

})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('RetrievePasswordController', RetrievePasswordController);

    RetrievePasswordController.$inject = ['$scope', 'httpService', 'alertService', '$timeout', 'routerStateService'];

    function RetrievePasswordController($scope, httpService, alertService, $timeout, routerStateService) {
        var vm = this;
        var timer = null;
        var regExpPhone = /^[0-9]{11}$/;
        var regExpEmail = /^.+@[^.].*.[a-z]{2,10}$/;

        vm.type = 'phone';
        vm.phone = '';
        vm.phoneCode = '';
        vm.email = '';
        vm.emailCode = '';
        vm.change = change;
        vm.flag = false;
        vm.second = '';
        vm.retrievePhone = retrievePhone;
        vm.retrieveEmail = retrieveEmail;
        vm.getPhoneCode = getPhoneCode;
        vm.getEmailCode = getEmailCode;

        function change(type) {
            vm.type = type;
            vm.flag = false;
            clearInterval(timer);
            vm.second = '';
        }

        function retrievePhone() {
            if (validatePhone()) {
                retrieve({
                    url: 'user/forgot_password/mobile',
                    address: vm.phone,
                    verify_code: vm.phoneCode
                });
            }
        }

        function retrieveEmail() {
            if (validateEmail()) {
                retrieve({
                    url: 'user/forgot_password/email',
                    address: vm.email,
                    verify_code: vm.emailCode
                });
            }
        }

        function retrieve(ops) {
            httpService.post(ops.url, {
                address: ops.address,
                verify_code: ops.verify_code
            }).then(function (result) {
                if (result.data.success) {
                    routerStateService.go('reset-password', {token: result.data.token});
                } else if (result.data.errorResponse) {
                    showPrompt(result.data.errorResponse.errMsg);
                }
            });
        }

        function validatePhone() {
            if (!regExpPhone.test($.trim(vm.phone))) {
                showPrompt('请输入正确的手机号码！');
                return false;
            }

            return true;
        }

        function validateEmail() {
            if (!regExpEmail.test($.trim(vm.email))) {
                showPrompt('请输入正确的邮箱！');
                return false;
            }

            return true;
        }

        function getPhoneCode() {
            getCode({
                validate: regExpPhone,
                url: 'verification/mobile',
                address: vm.phone
            });
        }

        function getEmailCode() {
            getCode({
                validate: regExpEmail,
                url: 'verification/email',
                address: vm.email
            });
        }

        function getCode(ops) {
            if (ops.validate.test($.trim(ops.address))) {
                vm.flag = true;
                httpService.post(ops.url, {
                    address: ops.address,
                    need_user: true
                }).then(function (result) {
                    if (result.data.success) {
                        var s = result.data.resendInterval;

                        vm.second = '（' + s + '）';
                        timer = setInterval(function () {
                            s--;
                            if (s < 0) {
                                clearInterval(timer);
                                vm.second = '';
                                vm.flag = false;
                            } else {
                                vm.second = '（' + s + '）';
                            }
                            $scope.$digest();

                        }, 1000);
                    } else if (result.data.errorResponse) {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            } else {
                if (vm.type === 'phone') {
                    showPrompt('请输入正确的手机号码！');
                } else if (vm.type === 'email') {
                    showPrompt('请输入正确的邮箱！');
                }
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('SecuritySettingsController', SecuritySettingsController);

    SecuritySettingsController.$inject = ['routerStateService', 'authService'];

    function SecuritySettingsController(routerStateService, authService) {
        var vm = this;
        var user = null;
        vm.bindEmail = false;
        vm.bindMobile = false;
        vm.onChangePassword = onChangePassword;
        vm.onBindPhone = onBindPhone;
        vm.onBindEmail = onBindEmail;

        updateInfo();
        ////////////////

        function updateInfo() {
            user = authService.getAuthenticatedUser();
            vm.bindMobile = user.mobile ? true : false;
            vm.bindEmail = user.email ? true : false;
        }

        function onChangePassword() {
            routerStateService.go('change-password');
        }

        function onBindPhone() {
            routerStateService.go('bind-strategy', {
                type: 'phone',
                isBind: !user.mobile,
                typeValue: user.mobile
            });
        }

        function onBindEmail() {
            routerStateService.go('bind-strategy', {
                type: 'email',
                isBind: !user.email,
                typeValue: user.email
            });
        }

    }
})();

(function() {
    'use strict';

    angular
        .module('app.user')
        .controller('SettingsController', SettingsController);

    SettingsController.$inject = ['$ionicPlatform', 'alertService', 'appConf', 'modalManagerService', '$scope'];

    function SettingsController($ionicPlatform, alertService, appConf, modalManagerService, $scope) {
        var vm = this;
        vm.clearCache = clearCache;
        vm.feedBack = feedBack;
        vm.scoreMe = scoreMe;
        vm.showAbout = showAbout;
        // Hide feedBack and scoreMe features on Android
        vm.isAndroid = $ionicPlatform.is('Android');

        ////////////////

        function feedBack() {
            cordova.plugins.settingService.feedback({
                success: function (msg) {
                },
                failed: function (msg) {
                    alertService.autoShow(msg);
                },
                address: 'support@ourdts.com',
                subject: '意见反馈',
                body: ''
            });
        }

        function clearCache() {
            cordova.plugins.settingService.clearCache({
                success: function (msg) {
                    alertService.autoShow(msg);
                },
                failed: function (msg) {
                    alertService.autoShow(msg);
                }
            });
        }

        function scoreMe() {
            cordova.plugins.settingService.rateApp({
                success: function (msg) {
                },
                failed: function (msg) {
                    alertService.autoShow(msg);
                },
                url: appConf.iosAppStoreUrl
            });
        }

        function showAbout() {
            modalManagerService.showModal('about', $scope);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('TermsController', TermsController);

    TermsController.$inject = ['$scope', '$sce', 'modalManagerService', 'httpService'];

    function TermsController($scope, $sce, modalManagerService, httpService) {
        var vm = this;
        vm.closeModal = closeModal;
        vm.targetURL = '';

        init();

        ////////////////

        function init() {
            var promise = httpService.get('terms_url');
            promise.then(function (result) {
                vm.targetURL = $sce.trustAsResourceUrl(result.data.url);
            });
        }

        function closeModal() {
            modalManagerService.closeModalByName('terms', $scope);
        }
    }

})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('UserAddressListController', UserAddressListController);

    UserAddressListController.$inject = ['httpService', 'alertService', '$timeout', 'routerStateService', '$stateParams', '$ionicHistory', '$rootScope'];

    function UserAddressListController(httpService, alertService, $timeout, routerStateService, $stateParams, $ionicHistory, $rootScope) {
        var vm = this;
        var products = null;
        vm.addressList = [];
        vm.select = select;
        vm.deleteAddress = deleteAddress;
        vm.updateAddress = updateAddress;
        vm.updateDefaultAddress = updateDefaultAddress;
        vm.createAddress = createAddress;
        vm.currentIsDefault = null;
        vm.defaultAddress = {};
        vm.showSetDefaultBtn = $stateParams.address_id == null;
        fetchAddressList();

        function fetchAddressList() {
            var promise = httpService.get('user/address_list');
            promise.then(function (result) {
                vm.addressList = result.data.addressList || [];
                if (vm.showSetDefaultBtn) {
                    angular.forEach(vm.addressList, function (item) {
                        if (item.isDefault) {
                            vm.defaultAddress = item;
                        }
                    });
                } else {
                    products = $stateParams.products;
                    angular.forEach(vm.addressList, function (item) {
                        console.debug($stateParams.address_id);
                        console.debug(item.id);
                        console.debug('~~~~~');
                        if (item.isDefault) {
                            vm.defaultAddress = item;
                            item.__isSelect__ = true;
                            if ($stateParams.address_id === '' || $stateParams.address_id === item.id) {
                                item.__isSelect__ = true;
                            } else {
                                item.__isSelect__ = false;
                            }
                        } else if ($stateParams.address_id === item.id) {
                            item.__isSelect__ = true;
                        }
                    });
                }
            });
        }

        function createAddress() {
            routerStateService.go('user-address', {address: null});
        }

        function select(item) {
            if (!vm.showSetDefaultBtn) {
                $ionicHistory.goBack();
                $timeout(function () {
                    $rootScope.$broadcast('checkout:refreshAddress', item, products);
                }, 10);
            } else {
                vm.currentIsDefault = item;
                updateDefaultAddress();
            }
        }

        function deleteAddress(item) {
            if (vm.defaultAddress.id == item.id) {
                showPrompt('默认地址不能删除!');
                return;
            }
            if (vm.currentIsDefault && vm.currentIsDefault.id === item.id) {
                vm.currentIsDefault = null;
            }
            var promise = httpService.post('user/del_address/' + item.id);
            promise.then(function (result) {
                if (result.data.success) {
                    fetchAddressList();
                } else if (result.data.errorRespones) {
                    showPrompt(result.data.errorRespones.errMsg);
                }
            });
        }

        function updateAddress(item) {
            routerStateService.go('user-address', {address: item});
        }

        function updateDefaultAddress() {
            if (vm.currentIsDefault) {
                var promise = httpService.post('user/set_default_address/' + vm.currentIsDefault.id);
                promise.then(function (result) {
                    if (result.data.success) {
                        vm.defaultAddress.isDefault = false;
                        vm.currentIsDefault.isDefault = true;
                        vm.defaultAddress = vm.currentIsDefault;
                        showPrompt('设置默认地址成功!');
                    } else if (result.data.errorRespones) {
                        showPrompt(result.data.errorRespones.errMsg);
                    }
                });
            } else {
                showPrompt('请选择你要设置的默认地址!');
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('UserAddressController', UserAddressController);

    UserAddressController.$inject = [
        '$scope', 'httpService', 'alertService', '$rootScope', '$timeout',
        '$ionicHistory', '$stateParams', 'modalManagerService'
    ];

    function UserAddressController($scope, httpService, alertService, $rootScope,
                                   $timeout, $ionicHistory, $stateParams, modalManagerService) {
        var vm = this;
        vm.address = $stateParams.address;
        vm.title = $stateParams.address != null ? '编辑收货地址' : '新建收货地址';
        vm.onDeliveryAreaClick = onDeliveryAreaClick;
        vm.saveAddress = saveAddress;

        ////////////////

        function onDeliveryAreaClick() {
            if (vm.address) {
                $scope.defaultArea = {
                    province: {
                        id: vm.address.provinceID,
                        name: vm.address.province
                    },
                    city: {
                        id: vm.address.cityID,
                        name: vm.address.city
                    },
                    district: {
                        id: vm.address.districtID,
                        name: vm.address.district
                    }
                };
            } else {
                $scope.defaultArea = null;
            }
            $scope.selectedArea = null;
            $scope.modalTitle = '配送所在地区';
            modalManagerService.showModal('userDeliveryArea', $scope, {
                onHidden: function (modal) {
                    if (!modal.scope.selectedArea) {
                        return;
                    }
                    var area = modal.scope.selectedArea;
                    vm.address = angular.extend({}, vm.address, {
                        districtID: area.district.id,
                        district: area.district.name,
                        cityID: area.city.id,
                        city: area.city.name,
                        provinceID: area.province.id,
                        province: area.province.name
                    });
                    modal.scope.selectedArea = null;
                }
            });
        }

        function saveAddress() {
            var promise = null;
            if (validateAddress()) {
                if ($stateParams.address) {
                    promise = httpService.post('user/edit_address/' + vm.address.id, {
                        address_id: vm.address.id,
                        recipient: vm.address.recipient,
                        phone: vm.address.phone,
                        district_id: vm.address.districtID,
                        address_desc: vm.address.addressDesc,
                        is_default: vm.address.isDefault
                    });
                } else {
                    promise = httpService.post('user/add_address', {
                        recipient: vm.address.recipient,
                        phone: vm.address.phone,
                        district_id: vm.address.districtID,
                        address_desc: vm.address.addressDesc
                    });
                }
                promise.then(function (result) {
                    if (!$stateParams.address && $stateParams.type) {
                        if (result.data.success) {
                            httpService.get('user/address_list')
                                .then(function (obj) {
                                    var addressList = obj.data.addressList || [];
                                    $.each(addressList, function (index, item) {
                                        if (result.data.userAddress.id === item.id) {
                                            $ionicHistory.goBack(-2);
                                            $timeout(function () {
                                                $rootScope.$broadcast('checkout:refreshAddress', item);
                                            }, 10);
                                        }
                                    });
                                });
                        } else {
                            showPrompt(result.data.errorRespones.errMsg);
                        }
                    } else {
                        if (result.data.success) {
                            $ionicHistory.goBack();
                        } else if (result.data.errorRespones) {
                            showPrompt(result.data.errorRespones.errMsg);
                        }
                    }
                });
            }
        }

        function validateAddress() {
            if ($.trim(vm.address.recipient) === '') {
                showPrompt('收件人不能为空!');
                return false;
            }
            if ($.trim(vm.address.phone) === '') {
                showPrompt('联系方式不能为空！');
                return false;
            }
            if (!vm.address.districtID) {
                showPrompt('请选择所在地区!');
                return false;
            }
            if ($.trim(vm.address.addressDesc) === '') {
                showPrompt('详细地址不能为空!');
                return false;
            }
            return true;
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('CreditsController', CreditsController);

    CreditsController.$inject = ['httpService'];

    function CreditsController(httpService) {
        var vm = this;
        vm.myCredit = null;

        getCredits();

        function getCredits() {
            var promise = httpService.get('get_balance');
            promise.then(function(response) {
                vm.myCredit = response.data;
            });
        }
    }

})();

(function() {
    'use strict';

    angular
        .module('app.user')
        .controller('UserInfoController', UserInfoController);

    UserInfoController.$inject = ['httpService', 'appConst'];

    function UserInfoController(httpService, appConst) {
        var vm = this;
        vm.birth = null;
        vm.gender = null;
        vm.genderText = '性别';
        vm.genderOptions = [];
        vm.lastBirth = null;
        vm.lastGender = null;
        vm.updateInfo = updateInfo;
        vm.change = change;

        readInfo();
        ////////////////

        function readInfo() {
            vm.genderOptions = [{
                id: appConst.USER_GENDER.UNKNOWN,
                label: '保密'
            }, {
                id: appConst.USER_GENDER.MALE,
                label: '男'
            }, {
                id: appConst.USER_GENDER.FEMALE,
                label: '女'
            }];

            var user_profile = httpService.get('user/profile');
            user_profile.then(function(result) {
                var user_info = result.data.userProfile.user_info;
                if (user_info) {
                    vm.gender = vm.lastGender = user_info.gender;
                    vm.birth = vm.lastBirth = new Date(user_info.birth);
                }
                change();
            });
        }


        function updateInfo() {
            if (vm.lastBirth === vm.birth && vm.lastGender === vm.gender){
                return;
            }

            var reqData = {};
            if (vm.birth !== null) {
                reqData.birth = vm.birth.getFullYear() + '-' +
                    (vm.birth.getMonth() + 1) + '-' + vm.birth.getDate();
            }
            if (vm.gender !== null) {
                reqData.gender = vm.gender;
            }
            var promise = httpService.post('user/edit_profile', reqData);
            promise.then(function(result) {
                if (result.data.success === true) {
                    vm.lastBirth = vm.birth;
                    vm.lastGender = vm.gender;
                }
            });
        }

        function change() {
            if (vm.gender === 0) {
                vm.genderText = '保密';
            } else if (vm.gender === 1) {
                vm.genderText = '男';
            } else if (vm.gender === 2) {
                vm.genderText = '女';
            } else {
                vm.genderText = '性别';
            }
        }
    }
})();

(function () {
    'use strict';

    angular
        .module('app.utils')
        .controller('DeliveryAreaListController', DeliveryAreaListController);

    DeliveryAreaListController.$inject = ['httpService', '$scope', 'modalManagerService'];

    function DeliveryAreaListController(httpService, $scope, modalManagerService) {
        var vm = this;
        vm.areaList = [];
        vm.goBack = goBack;
        vm.onAreaSelected = onAreaSelected;
        vm.closeModal = closeModal;
        vm.selectedAreaID = null;
        vm.showType = 'province';
        vm.showBackBtn = vm.showType === 'city' || vm.showType === 'district';

        $scope.$on('deliveryArea:show', function(event, modal) {
            vm.defaultArea = modal.scope.defaultArea;
            vm.selectedArea = modal.scope.selectedArea;
            vm.title = modal.scope.modalTitle;
            vm.modal = modal;

            fetchProvinceList();
        });

        ////////////////

        function fetchCityList(province) {
            var promise = httpService.get('city_list/' + province.id);
            promise.then(function (result) {
                var cityList = result.data.cityList;
                vm.areaList = [];
                angular.forEach(cityList, function (city) {
                    vm.areaList.push({
                        id: city.cityID,
                        name: city.cityName
                    });
                });
                vm.showType = 'city';
                var city = vm.areaList[0];
                if (vm.defaultArea && vm.defaultArea.province && province.id === vm.defaultArea.province.id) {
                    city = vm.defaultArea.city;
                }
                if (vm.selectedArea && vm.selectedArea.city && province.id === vm.selectedArea.province.id) {
                    city = vm.selectedArea.city;
                }
                if (vm.selectedArea) {
                    vm.selectedArea.province = province;
                } else {
                    vm.selectedArea = {
                        province: province
                    };
                }
                vm.selectedAreaID = city.id;
            });
        }

        function fetchDistrictList(city) {
            var promise = httpService.get('district_list/' + city.id);
            promise.then(function (result) {
                var districtList = result.data.districtList;
                vm.areaList = [];
                angular.forEach(districtList, function(district) {
                    vm.areaList.push({
                        id: district.districtID,
                        name: district.districtName
                    });
                });
                vm.showType = 'district';
                vm.selectedArea.city = city;
                var district = vm.areaList[0];
                if (vm.defaultArea && vm.defaultArea.city && city.id === vm.defaultArea.city.id) {
                    district = vm.defaultArea.district;
                }
                vm.selectedAreaID = district.id;
            });
        }

        function fetchProvinceList() {
            var promise = httpService.get('province_list');
            promise.then(function (result) {
                var provinceList = result.data.provinceList;
                vm.areaList = [];
                angular.forEach(provinceList, function (province) {
                    vm.areaList.push({
                        id: province.provinceID,
                        name: province.provinceName
                    });
                });
                vm.showType = 'province';
                var province = vm.areaList[0];
                if (vm.defaultArea && vm.defaultArea.province.id) {
                    province = vm.defaultArea.province;
                }
                if (vm.selectedArea && vm.selectedArea.province) {
                    province = vm.selectedArea.province;
                }
                vm.selectedAreaID = province.id;
            });
        }

        function onAreaSelected(area) {
            if (vm.showType === 'province') {
                fetchCityList(area);
            } else if (vm.showType === 'city') {
                fetchDistrictList(area);
            } else if (vm.showType === 'district') {
                modalManagerService.closeModal(vm.modal, function(modal) {
                    vm.selectedArea.district = area;
                    modal.scope.selectedArea = vm.selectedArea;
                });
            }
        }

        function goBack() {
            if (vm.showType === 'city') {
                fetchProvinceList();
            } else if (vm.showType === 'district') {
                fetchCityList(vm.selectedArea.province);
            }
        }

        function closeModal() {
            modalManagerService.closeModal(vm.modal);
        }
    }
})();

/*
* Adapted from Swipe.js 2.0
*
* Brad Birdsall
* Copyright 2013, MIT License
*
*/
(function (ionic) {
    'use strict';

    if (ionic.views.Slider) { delete ionic.views.Slider }
    ionic.views.Slider = ionic.views.View.inherit({
        initialize: function (options) {
            var slider = this;
            var ionic_continuous_flag = options.continuous;

            // utilities
            var noop = function () {
            }; // simple no operation function
            var offloadFn = function (fn) {
                setTimeout(fn || noop, 0);
            }; // offload a functions execution

            // check browser capabilities
            var browser = {
                addEventListener: !!window.addEventListener,
                touch: ('ontouchstart' in window) || window.DocumentTouch && document instanceof DocumentTouch,
                transitions: (function (temp) {
                    var props = ['transitionProperty', 'WebkitTransition', 'MozTransition', 'OTransition', 'msTransition'];
                    for (var i in props) if (temp.style[props[i]] !== undefined) return true;
                    return false;
                })(document.createElement('swipe'))
            };


            var container = options.el;

            // quit if no root element
            if (!container) return;
            var element = container.children[0];
            var slides, slidePos, width, length;
            options = options || {};
            var index = parseInt(options.startSlide, 10) || 0;
            var speed = options.speed || 300;
            options.continuous = options.continuous !== undefined ? options.continuous : true;

            function setup() {
                options.continuous = ionic_continuous_flag;
                // do not setup if the container has no width
                if (!container.offsetWidth) {
                    return;
                }

                // cache slides
                slides = element.children;
                length = slides.length;

                // set continuous to false if only one slide
                if (slides.length < 2) options.continuous = false;

                //special case if two slides
                if (browser.transitions && options.continuous && slides.length < 3) {
                    element.appendChild(slides[0].cloneNode(true));
                    element.appendChild(element.children[1].cloneNode(true));
                    slides = element.children;
                }

                // create an array to store current positions of each slide
                slidePos = new Array(slides.length);

                // determine width of each slide
                width = container.offsetWidth || container.getBoundingClientRect().width;

                element.style.width = (slides.length * width) + 'px';

                // stack elements
                var pos = slides.length;
                while (pos--) {

                    var slide = slides[pos];

                    slide.style.width = width + 'px';
                    slide.setAttribute('data-index', pos);

                    if (browser.transitions) {
                        slide.style.left = (pos * -width) + 'px';
                        move(pos, index > pos ? -width : (index < pos ? width : 0), 0);
                    }

                }

                // reposition elements before and after index
                if (options.continuous && browser.transitions) {
                    move(circle(index - 1), -width, 0);
                    move(circle(index + 1), width, 0);
                }

                if (!browser.transitions) element.style.left = (index * -width) + 'px';

                container.style.visibility = 'visible';

                options.slidesChanged && options.slidesChanged();
            }

            function prev(slideSpeed) {

                if (options.continuous) slide(index - 1, slideSpeed);
                else if (index) slide(index - 1, slideSpeed);

            }

            function next(slideSpeed) {

                if (options.continuous) slide(index + 1, slideSpeed);
                else if (index < slides.length - 1) slide(index + 1, slideSpeed);

            }

            function circle(index) {

                // a simple positive modulo using slides.length
                return (slides.length + (index % slides.length)) % slides.length;

            }

            function slide(to, slideSpeed) {

                // do nothing if already on requested slide
                if (index == to) return;

                if (browser.transitions) {

                    var direction = Math.abs(index - to) / (index - to); // 1: backward, -1: forward

                    // get the actual position of the slide
                    if (options.continuous) {
                        var naturalDirection = direction;
                        direction = -slidePos[circle(to)] / width;

                        // if going forward but to < index, use to = slides.length + to
                        // if going backward but to > index, use to = -slides.length + to
                        if (direction !== naturalDirection) to = -direction * slides.length + to;

                    }

                    var diff = Math.abs(index - to) - 1;

                    // move all the slides between index and to in the right direction
                    while (diff--) move(circle((to > index ? to : index) - diff - 1), width * direction, 0);

                    to = circle(to);

                    move(index, width * direction, slideSpeed || speed);
                    move(to, 0, slideSpeed || speed);

                    if (options.continuous) move(circle(to - direction), -(width * direction), 0); // we need to get the next in place

                } else {

                    to = circle(to);
                    animate(index * -width, to * -width, slideSpeed || speed);
                    //no fallback for a circular continuous if the browser does not accept transitions
                }

                index = to;
                offloadFn(options.callback && options.callback(index, slides[index]));
            }

            function move(index, dist, speed) {

                translate(index, dist, speed);
                slidePos[index] = dist;

            }

            function translate(index, dist, speed) {

                var slide = slides[index];
                var style = slide && slide.style;

                if (!style) return;

                style.webkitTransitionDuration =
                    style.MozTransitionDuration =
                        style.msTransitionDuration =
                            style.OTransitionDuration =
                                style.transitionDuration = speed + 'ms';

                style.webkitTransform = 'translate(' + dist + 'px,0)' + 'translateZ(0)';
                style.msTransform =
                    style.MozTransform =
                        style.OTransform = 'translateX(' + dist + 'px)';

            }

            function animate(from, to, speed) {

                // if not an animation, just reposition
                if (!speed) {

                    element.style.left = to + 'px';
                    return;

                }

                var start = +new Date();

                var timer = setInterval(function () {

                    var timeElap = +new Date() - start;

                    if (timeElap > speed) {

                        element.style.left = to + 'px';

                        if (delay) begin();

                        options.transitionEnd && options.transitionEnd.call(event, index, slides[index]);

                        clearInterval(timer);
                        return;

                    }

                    element.style.left = (( (to - from) * (Math.floor((timeElap / speed) * 100) / 100) ) + from) + 'px';

                }, 4);

            }

            // setup auto slideshow
            var delay = options.auto || 0;
            var interval;

            function begin() {

                interval = setTimeout(next, delay);

            }

            function stop() {

                delay = options.auto || 0;
                clearTimeout(interval);

            }


            // setup initial vars
            var start = {};
            var delta = {};
            var isScrolling;

            // setup event capturing
            var events = {

                handleEvent: function (event) {
                    if (event.type == 'mousedown' || event.type == 'mouseup' || event.type == 'mousemove') {
                        event.touches = [{
                            pageX: event.pageX,
                            pageY: event.pageY
                        }];
                    }

                    switch (event.type) {
                        case 'mousedown':
                            this.start(event);
                            break;
                        case 'touchstart':
                            this.start(event);
                            break;
                        case 'touchmove':
                            this.touchmove(event);
                            break;
                        case 'mousemove':
                            this.touchmove(event);
                            break;
                        case 'touchend':
                            offloadFn(this.end(event));
                            break;
                        case 'mouseup':
                            offloadFn(this.end(event));
                            break;
                        case 'webkitTransitionEnd':
                        case 'msTransitionEnd':
                        case 'oTransitionEnd':
                        case 'otransitionend':
                        case 'transitionend':
                            offloadFn(this.transitionEnd(event));
                            break;
                        case 'resize':
                            offloadFn(setup);
                            break;
                    }

                    if (options.stopPropagation) event.stopPropagation();

                },
                start: function (event) {

                    var touches = event.touches[0];

                    // measure start values
                    start = {

                        // get initial touch coords
                        x: touches.pageX,
                        y: touches.pageY,

                        // store time to determine touch duration
                        time: +new Date()

                    };

                    // used for testing first move event
                    isScrolling = undefined;

                    // reset delta and end measurements
                    delta = {};

                    // attach touchmove and touchend listeners
                    if (browser.touch) {
                        element.addEventListener('touchmove', this, false);
                        element.addEventListener('touchend', this, false);
                    } else {
                        element.addEventListener('mousemove', this, false);
                        element.addEventListener('mouseup', this, false);
                        document.addEventListener('mouseup', this, false);
                    }
                },
                touchmove: function (event) {

                    // ensure swiping with one touch and not pinching
                    // ensure sliding is enabled
                    if (event.touches.length > 1 ||
                        event.scale && event.scale !== 1 ||
                        slider.slideIsDisabled) {
                        return;
                    }

                    if (options.disableScroll) event.preventDefault();

                    var touches = event.touches[0];

                    // measure change in x and y
                    delta = {
                        x: touches.pageX - start.x,
                        y: touches.pageY - start.y
                    };

                    // determine if scrolling test has run - one time test
                    if (typeof isScrolling == 'undefined') {
                        isScrolling = !!( isScrolling || Math.abs(delta.x) < Math.abs(delta.y) );
                    }

                    // if user is not trying to scroll vertically
                    if (!isScrolling) {

                        // prevent native scrolling
                        event.preventDefault();

                        // stop slideshow
                        stop();

                        // increase resistance if first or last slide
                        if (options.continuous) { // we don't add resistance at the end

                            translate(circle(index - 1), delta.x + slidePos[circle(index - 1)], 0);
                            translate(index, delta.x + slidePos[index], 0);
                            translate(circle(index + 1), delta.x + slidePos[circle(index + 1)], 0);

                        } else {

                            delta.x =
                                delta.x /
                                ( (!index && delta.x > 0 ||         // if first slide and sliding left
                                index == slides.length - 1 &&     // or if last slide and sliding right
                                delta.x < 0                       // and if sliding at all
                                ) ?
                                    ( Math.abs(delta.x) / width + 1 )      // determine resistance level
                                    : 1 );                                 // no resistance if false

                            // translate 1:1
                            translate(index - 1, delta.x + slidePos[index - 1], 0);
                            translate(index, delta.x + slidePos[index], 0);
                            translate(index + 1, delta.x + slidePos[index + 1], 0);
                        }

                        options.onDrag && options.onDrag();
                    }

                },
                end: function () {

                    // measure duration
                    var duration = +new Date() - start.time;

                    // determine if slide attempt triggers next/prev slide
                    var isValidSlide =
                        Number(duration) < 250 &&         // if slide duration is less than 250ms
                        Math.abs(delta.x) > 20 ||         // and if slide amt is greater than 20px
                        Math.abs(delta.x) > width / 2;      // or if slide amt is greater than half the width

                    // determine if slide attempt is past start and end
                    var isPastBounds = (!index && delta.x > 0) ||      // if first slide and slide amt is greater than 0
                        (index == slides.length - 1 && delta.x < 0); // or if last slide and slide amt is less than 0

                    if (options.continuous) isPastBounds = false;

                    // determine direction of swipe (true:right, false:left)
                    var direction = delta.x < 0;

                    // if not scrolling vertically
                    if (!isScrolling) {

                        if (isValidSlide && !isPastBounds) {

                            if (direction) {

                                if (options.continuous) { // we need to get the next in this direction in place

                                    move(circle(index - 1), -width, 0);
                                    move(circle(index + 2), width, 0);

                                } else {
                                    move(index - 1, -width, 0);
                                }

                                move(index, slidePos[index] - width, speed);
                                move(circle(index + 1), slidePos[circle(index + 1)] - width, speed);
                                index = circle(index + 1);

                            } else {
                                if (options.continuous) { // we need to get the next in this direction in place

                                    move(circle(index + 1), width, 0);
                                    move(circle(index - 2), -width, 0);

                                } else {
                                    move(index + 1, width, 0);
                                }

                                move(index, slidePos[index] + width, speed);
                                move(circle(index - 1), slidePos[circle(index - 1)] + width, speed);
                                index = circle(index - 1);

                            }

                            options.callback && options.callback(index, slides[index]);

                        } else {

                            if (options.continuous) {

                                move(circle(index - 1), -width, speed);
                                move(index, 0, speed);
                                move(circle(index + 1), width, speed);

                            } else {

                                move(index - 1, -width, speed);
                                move(index, 0, speed);
                                move(index + 1, width, speed);
                            }

                        }

                    }

                    // kill touchmove and touchend event listeners until touchstart called again
                    if (browser.touch) {
                        element.removeEventListener('touchmove', events, false);
                        element.removeEventListener('touchend', events, false);
                    } else {
                        element.removeEventListener('mousemove', events, false);
                        element.removeEventListener('mouseup', events, false);
                        document.removeEventListener('mouseup', events, false);
                    }

                    options.onDragEnd && options.onDragEnd();
                },
                transitionEnd: function (event) {

                    if (parseInt(event.target.getAttribute('data-index'), 10) == index) {

                        if (delay) begin();

                        options.transitionEnd && options.transitionEnd.call(event, index, slides[index]);

                    }

                }

            };

            // Public API
            this.update = function () {
                setTimeout(setup);
            };
            this.setup = function () {
                setup();
            };

            this.loop = function (value) {
                if (arguments.length) options.continuous = !!value;
                return options.continuous;
            };

            this.enableSlide = function (shouldEnable) {
                if (arguments.length) {
                    this.slideIsDisabled = !shouldEnable;
                }
                return !this.slideIsDisabled;
            };

            this.slide = this.select = function (to, speed) {
                // cancel slideshow
                stop();

                slide(to, speed);
            };

            this.prev = this.previous = function () {
                // cancel slideshow
                stop();

                prev();
            };

            this.next = function () {
                // cancel slideshow
                stop();

                next();
            };

            this.stop = function () {
                // cancel slideshow
                stop();
            };

            this.start = function () {
                begin();
            };

            this.autoPlay = function (newDelay) {
                if (!delay || delay < 0) {
                    stop();
                } else {
                    delay = newDelay;
                    begin();
                }
            };

            this.currentIndex = this.selected = function () {
                // return current index position
                return index;
            };

            this.slidesCount = this.count = function () {
                // return total number of slides
                return length;
            };

            this.kill = function () {
                // cancel slideshow
                stop();

                // reset element
                element.style.width = '';
                element.style.left = '';

                // reset slides so no refs are held on to
                slides && (slides = []);

                // removed event listeners
                if (browser.addEventListener) {

                    // remove current event listeners
                    element.removeEventListener('touchstart', events, false);
                    element.removeEventListener('webkitTransitionEnd', events, false);
                    element.removeEventListener('msTransitionEnd', events, false);
                    element.removeEventListener('oTransitionEnd', events, false);
                    element.removeEventListener('otransitionend', events, false);
                    element.removeEventListener('transitionend', events, false);
                    window.removeEventListener('resize', events, false);

                }
                else {

                    window.onresize = null;

                }
            };

            this.load = function () {
                // trigger setup
                setup();

                // start auto slideshow if applicable
                if (delay) begin();


                // add event listeners
                if (browser.addEventListener) {

                    // set touchstart event on element
                    if (browser.touch) {
                        element.addEventListener('touchstart', events, false);
                    } else {
                        element.addEventListener('mousedown', events, false);
                    }

                    if (browser.transitions) {
                        element.addEventListener('webkitTransitionEnd', events, false);
                        element.addEventListener('msTransitionEnd', events, false);
                        element.addEventListener('oTransitionEnd', events, false);
                        element.addEventListener('otransitionend', events, false);
                        element.addEventListener('transitionend', events, false);
                    }

                    // set resize event on window
                    window.addEventListener('resize', events, false);

                } else {

                    window.onresize = function () {
                        setup();
                    }; // to play nice with old IE

                }
            };

        }
    });

})(ionic);

(function() {
    'use strict';

    angular
        .module('app')
        .controller('LocationController', LocationController);

    LocationController.$inject = ['$rootScope', 'currentCityService', '$window'];

    function LocationController($rootScope, currentCityService, $window) {
        var vm = this;
        vm.changeLocation = changeLocation;
        vm.currentLocation = '全国';
        vm.locations = [];

        init();
        $rootScope.$on('currentCity:changed', loadLocation);

        ////////////////

        function init() {
            currentCityService.fetchCities()
                .then(function(result) {
                vm.locations = result.data.provinceListWithCities;
            });
            loadLocation();
        }

        function loadLocation() {
            var city = currentCityService.getCurrentCity();
            if (city) {
                vm.currentLocation = city.name;
            } else {
                vm.currentLocation = '全国';
            }
        }

        function changeLocation(city) {
            if (city) {
                currentCityService.setCurrentCity(city);
                $window.location.href = '#/tab/dashboard';
            }
        }
    }
})();
