(function () {
    'use strict';

    /* Shared modules */
    angular
        .module('app.core', [
            /* Cross-app modules */
            'app.config',
            'app.service',

            /* 3rd-party modules */
            'ionic'
        ]);

    /* Cross-app modules */
    angular
        .module('app.config', []);
    angular
        .module('app.service', []);

    /* Component modules */
    angular
        .module('app.component', []);

    /* Feature modules */
    angular
        .module('app.dashboard', []);
    angular
        .module('app.order', []);
    angular
        .module('app.user', []);
    angular
        .module('app.utils', []);


    /* Main module */
    angular
        .module('app', [
            /* Share modules */
            'app.core',

            /* Component modules */
            'app.component',

            /* Feature modules */
            'app.dashboard',
            'app.order',
            'app.user',
            'app.utils'
        ]);

    //////////

    angular
        .module('app')
        .run(runBlock);

    runBlock.$inject = [
        '$rootScope', '$ionicPlatform', '$ionicPopup', '$ionicLoading', '$ionicTabsDelegate',
        'routerStateService', 'authService', 'cartService'
    ];

    function runBlock($rootScope, $ionicPlatform, $ionicPopup, $ionicLoading, $ionicTabsDelegate,
                      routerStateService, authService, cartService) {
        $ionicPlatform.ready(function () {
            // Hide the accessory bar by default (remove this to show the
            // accessory bar above the keyboard for form inputs)
            if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            }
            if (window.StatusBar) {
                // org.apache.cordova.statusbar required
                StatusBar.styleLightContent();
            }
            if (navigator.splashscreen) {
                setTimeout(function () {
                    navigator.splashscreen.hide();
                }, 3000);
            }
        });

        $rootScope.$on('error:show', function (event, args) {
            if (!$rootScope.isAPIErrorAlertShowing) {
                $rootScope.isAPIErrorAlertShowing = true;
                var promise = $ionicPopup.alert({
                    title: args.message
                });
                promise.then(function () {
                    $rootScope.isAPIErrorAlertShowing = false;
                    if (args.nextState) {
                        routerStateService.go(args.nextState);
                    }
                });
            }
        });

        $rootScope.$on('loading:show', function () {
            if (!$rootScope.loadingOverlay) {
                $rootScope.loadingOverlay = $ionicLoading;
            }
            $rootScope.loadingOverlay.show();
        });
        $rootScope.$on('loading:hide', function () {
            if ($rootScope.loadingOverlay) {
                $rootScope.loadingOverlay.hide();
            }
        });

        $rootScope.$on('$stateChangeStart', function (event, nextState, nextParams, currentState) {
            var isAuthorized = authService.isAuthorized();
            if ('data' in nextState && 'requireLogin' in nextState.data && !isAuthorized) {
                event.preventDefault();
                routerStateService.go('login');
            } else if (isAuthorized && 'data' in nextState && 'noLoginRequired' in nextState.data) {
                event.preventDefault();
                $ionicTabsDelegate.select($ionicTabsDelegate.selectedIndex());
            }
        });

        cartService.refreshCart();
        $rootScope.$on('cartService:refreshCart', cartService.refreshCart);
    }
})();
