(function () {
    'use strict';

    angular
        .module('app')
        .config(configure);

    configure.$inject = ['$stateProvider', '$urlRouterProvider', '$httpProvider', '$ionicConfigProvider'];

    function configure($stateProvider, $urlRouterProvider, $httpProvider, $ionicConfigProvider) {
        // Ionic uses AngularUI Router which uses the concept of states
        // Learn more here: https://github.com/angular-ui/ui-router
        // Set up the various states which the app can be in.
        // Each state's controller can be found in controllers.js
        $stateProvider
            // setup an abstract state for the tabs directive
            .state('tab', {
                url: '/tab',
                abstract: true,
                templateUrl: 'templates/layout/tabs.html'
            })

            // Each tab has its own nav history stack
            // Dashboard
            .state('tab.dashboard', {
                url: '/dashboard',
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/dashboard.html'
                    }
                }
            })
            .state('dashboard-locations', {
                url: '/locations',
                parent: 'tab',
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/locations.html'
                    }
                }
            })
            .state('dashboard-product', {
                url: '/product/:id',
                parent: 'tab',
                cache: false,
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/product.html'
                    }
                }
            })
            .state('dashboard-product-types', {
                url: '/product-types',
                parent: 'tab',
                cache: false,
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/product-types.html'
                    }
                }
            })
            .state('dashboard-product-list', {
                cache: false,
                url: '/product-list',
                parent: 'tab',
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/product-list.html'
                    }
                }
            })
            .state('dashboard-checkout', {
                url: '/checkout',
                parent: 'tab',
                params: {
                    'products': null,
                    'isDirectBuy': false
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/checkout.html'
                    }
                }
            })
            .state('dashboard-order-list', {
                cache: false,
                url: '/order-list',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/order-list.html'
                    }
                }
            })
            .state('dashboard-order-info', {
                cache: false,
                url: '/order-info',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/order-info.html'
                    }
                }
            })
            .state('dashboard-order-refund', {
                url: '/order-refund',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/order-refund.html'
                    }
                }
            })
            .state('dashboard-pay', {
                url: '/pay/:transId',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/pay.html'
                    }
                }
            })
            .state('dashboard-pay-success', {
                url: '/pay-success',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/pay-success.html'
                    }
                }
            })
            .state('dashboard-user-address-list', {
                url: '/user-address-list',
                parent: 'tab',
                params: {
                    'address_id': '',
                    'products': null
                },
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/user-address-list.html'
                    }
                }
            })
            .state('dashboard-user-address', {
                url: '/user-address',
                parent: 'tab',
                params: {
                    'address': null,
                    'type': 'dashboard'
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/user-address.html'
                    }
                }
            })
            .state('dashboard-login', {
                url: '/login',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/login.html'
                    }
                }
            })
            .state('dashboard-register', {
                url: '/register',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/register.html'
                    }
                }
            })
            .state('dashboard-retrieve-password', {
                url: '/retrieve-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/retrieve-password.html'
                    }
                }
            })
            .state('dashboard-reset-password', {
                url: '/reset-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                params: {token: ''},
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/reset-password.html'
                    }
                }
            })
            .state('dashboard-custom-invoice', {
                url: '/custom-invoice',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-dashboard': {
                        templateUrl: 'templates/pages/custom-invoice.html'
                    }
                }
            })

            // Cart
            .state('tab.cart', {
                url: '/cart',
                cache: false,
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/cart.html'
                    }
                }
            })
            .state('cart-product', {
                url: '/product/:id',
                parent: 'tab',
                cache: false,
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/product.html'
                    }
                }
            })
            .state('cart-checkout', {
                url: '/checkout',
                parent: 'tab',
                params: {'products': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/checkout.html'
                    }
                }
            })
            .state('cart-order-list', {
                cache: false,
                url: '/order-list',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/order-list.html'
                    }
                }
            })
            .state('cart-order-info', {
                cache: false,
                url: '/order-info',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/order-info.html'
                    }
                }
            })
            .state('cart-order-refund', {
                url: '/order-refund',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/order-refund.html'
                    }
                }
            })
            .state('cart-pay', {
                url: '/pay/:transId',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/pay.html'
                    }
                }
            })
            .state('cart-pay-success', {
                url: '/pay-success',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/pay-success.html'
                    }
                }
            })
            .state('cart-user-address-list', {
                url: '/user-address-list',
                parent: 'tab',
                params: {
                    'address_id': '',
                    'products': null
                },
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/user-address-list.html'
                    }
                }
            })
            .state('cart-user-address', {
                url: '/cart-user-address',
                parent: 'tab',
                params: {
                    'address': null,
                    'type': 'cart'
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/user-address.html'
                    }
                }
            })
            .state('cart-login', {
                url: '/login',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/login.html'
                    }
                }
            })
            .state('cart-register', {
                url: '/register',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/register.html'
                    }
                }
            })
            .state('cart-retrieve-password', {
                url: '/retrieve-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/retrieve-password.html'
                    }
                }
            })
            .state('cart-reset-password', {
                url: '/reset-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                params: {token: ''},
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/reset-password.html'
                    }
                }
            })
            .state('cart-custom-invoice', {
                url: '/custom-invoice',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-cart': {
                        templateUrl: 'templates/pages/custom-invoice.html'
                    }
                }
            })

            // Account
            .state('tab.account', {
                url: '/account',
                cache: false,
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/account.html'
                    }
                }
            })
            .state('account-settings', {
                url: '/settings',
                parent: 'tab',
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/settings.html',
                        controller: 'SettingsController as settings'
                    }
                }
            })
            .state('account-login', {
                url: '/login',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/login.html'
                    }
                }
            })
            .state('account-retrieve-password', {
                url: '/retrieve-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/retrieve-password.html'
                    }
                }
            })
            .state('account-reset-password', {
                url: '/reset-password',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                params: {token: ''},
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/reset-password.html'
                    }
                }
            })
            .state('account-register', {
                url: '/register',
                parent: 'tab',
                data: {
                    noLoginRequired: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/register.html'
                    }
                }
            })
            .state('account-user-address-list', {
                url: '/user-address-list',
                parent: 'tab',
                params: {'type': 'account'},
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/user-address-list.html'
                    }
                }
            })
            .state('account-user-address', {
                url: '/user-address',
                parent: 'tab',
                params: {
                    'address': null
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/user-address.html'
                    }
                }
            })
            .state('account-user-credits', {
                url: '/get-balance',
                parent: 'tab',
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/user-credits.html'
                    }
                }
            })
            .state('account-charge-redemption', {
                url: '/charge-redemption',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/charge-redemption.html'
                    }
                }
            })
            .state('account-userinfo', {
                url: '/userinfo',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/user-info.html'
                    }
                }
            })
            .state('account-security-settings', {
                url: '/security-settings',
                parent: 'tab',
                cache: false,
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/security-settings.html'
                    }
                }
            })
            .state('account-change-password', {
                url: '/change-password',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/change-password.html'
                    }
                }
            })
            .state('account-order-list', {
                cache: false,
                url: '/order-list',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/order-list.html'
                    }
                }
            })
            .state('account-order-info', {
                cache: false,
                url: '/order-info',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/order-info.html'
                    }
                }
            })
            .state('account-order-refund', {
                url: '/order-refund',
                parent: 'tab',
                params: {'orderId': null},
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/order-refund.html'
                    }
                }
            })
            .state('account-pay', {
                url: '/pay/:transId',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/pay.html'
                    }
                }
            })
            .state('account-pay-success', {
                url: '/pay-success',
                parent: 'tab',
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/pay-success.html'
                    }
                }
            })
            .state('account-bind-strategy', {
                url: '/bind-strategy',
                parent: 'tab',
                cache: false,
                params: {
                    'type': '',
                    'typeValue': '',
                    'isBind': false
                },
                data: {
                    requireLogin: true
                },
                views: {
                    'tab-account': {
                        templateUrl: 'templates/pages/bind-strategy.html'
                    }
                }
            });


        // if none of the above states are matched, use this as the fallback
        $urlRouterProvider.otherwise('/tab/dashboard');

        // Register http interceptor
        if (ArkenstoneAppSettings.debug) {
            $httpProvider.defaults.headers.common['X-Requested-With'] = 'XMLHttpRequest';
        }
        $httpProvider.defaults.withCredentials = true;
        $httpProvider.interceptors.push('httpInterceptor');

        // Platform configurations
        $ionicConfigProvider.backButton.text('').icon('ion-ios-arrow-left').previousTitleText(false);
        $ionicConfigProvider.navBar.alignTitle('center');
        $ionicConfigProvider.tabs.position('bottom').style('standard');
    }
})();
