(function () {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('ProductDetailController', ProductDetailController);

    ProductDetailController.$inject = [
        '$scope', 'httpService', '$stateParams', 'routerStateService',
        '$ionicScrollDelegate', '$timeout', '$ionicHistory', 'cartService',
        'authService', 'modalManagerService', 'alertService'
    ];

    function ProductDetailController($scope, httpService, $stateParams, routerStateService,
                                     $ionicScrollDelegate, $timeout, $ionicHistory, cartService,
                                     authService, modalManagerService, alertService) {
        var vm = this;
        vm.activeDetailTab = 'detail';
        vm.addToCart = addToCart;
        vm.cartSize = cartService.getTotalQuantity();
        vm.changeNumber = changeNumber;
        vm.deliveryCtrl = null;
        vm.deliveryModal = null;
        vm.detailImages = [];
        vm.hasMore = true;
        vm.loadDetails = loadDetails;
        vm.loadBasicInfo = loadBasicInfo;
        vm.onCheckout = onCheckout;
        vm.onDeliveryAreaClick = onDeliveryAreaClick;
        vm.product = null;
        vm.productReviews = [];
        vm.quantity = 1;
        vm.questionList = [];
        vm.refreshProgress = refreshProgress;
        vm.showQuestionBtn = authService.isAuthorized();
        vm.switchDetailTab = switchDetailTab;


        fetchProduct();
        $scope.$on('scrollTop', loadBasicInfo);

        ////////////////

        function addToCart() {
            cartService.addToCart(vm.product.id, vm.quantity, function () {
                vm.cartSize = cartService.getTotalQuantity();
                alertService.autoShow('恭喜，已添加至购物车。');
            });
        }

        function changeNumber(value) {
            vm.quantity = value;
        }

        function fetchProduct() {
            var promise = httpService.get('product/' + $stateParams.id);
            promise.then(function (result) {
                vm.product = result.data.product;
            }, function (error) {
                if (error.status === 404) {
                    $ionicHistory.goBack();
                }
            });
        }

        function loadBasicInfo() {
            $scope.$broadcast('scroll.refreshComplete');
            vm.hasMore = true;
            $ionicScrollDelegate.scrollTop();
        }

        function loadDetails() {
            if (vm.detailImages.length) {
                showDetails();
            } else {
                var promise = httpService.get('product_details/' + $stateParams.id);
                promise.then(function (result) {
                    vm.detailImages = result.data.detailImages;
                    showDetails();
                });
            }
        }

        function onCheckout() {
            var product = angular.extend({}, {
                quantity: vm.quantity,
                checked: true
            }, vm.product);
            routerStateService.go('checkout', {
                products: [product],
                isDirectBuy: true
            });
        }

        function onDeliveryAreaClick() {
            $scope.defaultArea = {
                province: {
                    id: vm.product.deliveryRange.provinceID,
                    name: vm.product.deliveryRange.provinceName
                },
                city: {
                    id: vm.product.deliveryRange.cityID,
                    name: vm.product.deliveryRange.cityName
                },
                district: {
                    id: vm.product.deliveryRange.districtID,
                    name: vm.product.deliveryRange.districtName
                }
            };
            $scope.selectedArea = null;
            $scope.modalTitle = '配送至';
            modalManagerService.showModal('productDeliveryArea', $scope, {
                onHidden: function (modal) {
                    if (!modal.scope.selectedArea) {
                        return;
                    }
                    var promise = httpService.get('product/' + vm.product.id + '/delivery_range/' + modal.scope.selectedArea.district.id);
                    promise.then(function (result) {
                        vm.product.deliveryRange = result.data.deliveryRange;
                        modal.scope.selectedArea = null;
                    });
                }
            });
        }

        function refreshProgress() {
            if (vm.product) {
                return {
                    width: vm.product.statsInfo.progress + '%'
                };
            }
        }

        function showDetails() {
            $scope.$broadcast('scroll.infiniteScrollComplete');
            vm.hasMore = false;
            $ionicScrollDelegate.scrollTop();
        }

        function showQuestionsAndAnswers() {
            if (!vm.questionList.length) {
                var promise = httpService.get('product_faq_list/' + $stateParams.id);
                promise.then(function (result) {
                    vm.questionList = result.data.faqList;
                });
            }
        }

        function showReviews() {
            if (!vm.productReviews.length) {
                var promise = httpService.get('product_review_list/' + $stateParams.id);
                promise.then(function (result) {
                    vm.productReviews = result.data.reviewList;
                });
            }
        }

        function switchDetailTab(tabType) {
            $ionicScrollDelegate.scrollTop();
            if (vm.activeDetailTab === tabType) {
                return;
            }
            vm.activeDetailTab = tabType;
            if (tabType === 'review') {
                showReviews();
            } else if (tabType === 'qa') {
                showQuestionsAndAnswers();
            } else {
                loadDetails();
            }
        }
    }
})();
