(function () {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('ProductListController', ProductListController);

    ProductListController.$inject = [
        '$scope', 'appConf', 'routerStateService', '$ionicViewSwitcher',
        '$ionicScrollDelegate', 'productSearchService'
    ];

    function ProductListController($scope, appConf, routerStateService, $ionicViewSwitcher,
                                   $ionicScrollDelegate, productSearchService) {
        var vm = this;
        vm.disableAddToCartBtnInProductList = appConf.disableAddToCartBtnInProductList;
        vm.hasMore = true;
        vm.page = 0;
        vm.productList = [];
        vm.orderBy = productSearchService.getParam('orderBy');
        vm.keyword = productSearchService.getParam('keyword');
        vm.goBack = goBack;
        vm.getProducts = getProducts;
        vm.queryByKeyword = queryByKeyword;
        vm.sortProducts = sortProducts;

        ////////////////

        function goBack() {
            productSearchService.resetParams();
            $ionicViewSwitcher.nextDirection('back');
            routerStateService.go('tab.dashboard');
        }

        function getProducts() {
            if (!vm.hasMore) {
                return;
            }

            var promise = productSearchService.search(vm.page);
            promise.then(function (result) {
                vm.productList = vm.productList.concat(result.data.productList);
                vm.hasMore = result.data.hasNextPage;
                vm.page = result.data.nextPage;
            }).catch(function () {
                vm.hasMore = false;
            }).finally(function () {
                $scope.$broadcast('scroll.infiniteScrollComplete');
            });
        }

        function queryByKeyword() {
            if (!vm.keyword || vm.keyword === productSearchService.getParam('keyword')) {
                return;
            }
            productSearchService.resetParams();
            productSearchService.setParam('keyword', vm.keyword);
            vm.orderBy = ['newest', true];
            if (routerStateService.getCurrentName().indexOf('product-list') === -1) {
                $ionicViewSwitcher.nextDirection('forward');
                routerStateService.go('product-list');
            } else {
                resetState();
            }
        }

        function resetState() {
            vm.productList = [];
            vm.page = 0;
            vm.hasMore = true;
            vm.orderBy = productSearchService.getParam('orderBy');
            vm.keyword = productSearchService.getParam('keyword');
            $ionicScrollDelegate.scrollTop();
        }

        function sortProducts(condition) {
            if (condition === vm.orderBy[0]) {
                vm.orderBy[1] = !vm.orderBy[1];
            } else {
                if (condition === 'newest') {
                    vm.orderBy = ['newest', true];
                } else {
                    vm.orderBy = ['left_quantity', false];
                }
            }
            productSearchService.setParam('orderBy', vm.orderBy);
            resetState();
        }
    }
})();
