(function() {
    'use strict';

    angular
        .module('app.dashboard')
        .controller('SpecialController', SpecialController);

    SpecialController.$inject = [
        '$rootScope', '$scope', 'httpService', 'routerStateService',
        '$ionicSlideBoxDelegate', 'currentCityService'];

    function SpecialController(
        $rootScope, $scope, httpService, routerStateService, $ionicSlideBoxDelegate, currentCityService) {
        var vm = this;
        vm.specialGroups = [];
        vm.onSpecialClick = onSpecialClick;

        fetchSpecial();
        // refresh products according to current city
        $rootScope.$on('currentCity:changed', fetchSpecial);
        // if vm.specialGroups is updated when this is not the current state,
        // $ionicSlideBoxDelegate.update doesn't work since it cannot know the
        // right size of the view. This ensures update is always called when
        // this becomes the current view.
        $scope.$on('$stateChangeSuccess', update);

        ////////////////

        function fetchSpecial() {
            var params = {};
            var city = currentCityService.getCurrentCity();
            if (city && city.id) {
                params.current_city = city.id;
            }
            var promise = httpService.get('recommend_product_list', params);
            promise.then(function(result) {
                vm.specialGroups = result.data.recommendProductList;
                update();
            });
        }

        function update(event, toState, toParams, fromState, fromParams) {
            if (!toState || toState.name === 'tab.dashboard') {
                $ionicSlideBoxDelegate.update();
                $ionicSlideBoxDelegate.slide(0);
                $ionicSlideBoxDelegate.start();
            }
        }

        function onSpecialClick(special) {
            routerStateService.go('product', {'id': special.productID});
        }
    }
})();
