(function () {
    'use strict';

    angular
        .module('app.order')
        .controller('CartController', CartController);

    CartController.$inject = ['$ionicPopup', 'routerStateService', 'cartService', 'alertService'];

    function CartController($ionicPopup, routerStateService, cartService, alertService) {
        var vm = this;
        vm.changeQuantity = changeQuantity;
        vm.checkAllProducts = checkAllProducts;
        vm.checkProduct = checkProduct;
        vm.deleteAllProducts = deleteAllProducts;
        vm.deleteProduct = deleteProduct;
        vm.goToProduct = goToProduct;
        vm.onCheckout = onCheckout;
        vm.hasAnyProduct = false;
        vm.isAllChecked = false;
        vm.isAnyChecked = false;
        vm.products = [];
        vm.totalPrice = 0;

        getProducts();

        ////////////////

        function changeQuantity(product, quantity) {
            if (product.quantity !== quantity) {
                cartService.updateCart(product.id, quantity, product.checked, getProducts);
            }
            event.cancelBubble = true;
        }

        function checkAllProducts() {
            var products = [];
            angular.forEach(vm.products, function (product) {
                products.push({
                    sale: product.id,
                    quantity: product.quantity,
                    checked: vm.isAllChecked
                });
            });
            cartService.bulkUpdateCart(products, getProducts);
        }

        function checkProduct(product) {
            cartService.updateCart(product.id, product.quantity, product.checked, getProducts);
            event.cancelBubble = true;
        }

        function deleteAllProducts() {
            var performDeleteAll = function () {
                var ids = [];
                angular.forEach(vm.products, function (product) {
                    if (product.checked) {
                        ids.push(product.id);
                    }
                });
                cartService.bulkRemoveFromCart(ids, getProducts);
            };

            $ionicPopup.confirm({
                title: '确认删除',
                template: '你确认要删除购物车里面选中的商品？',
                buttons: [
                    {
                        text: '删除',
                        type: 'button-assertive',
                        onTap: performDeleteAll
                    }, {
                        text: '取消',
                        type: 'button-balanced'
                    }
                ]
            });
        }

        function deleteProduct(product) {
            cartService.removeFromCart(product.id, getProducts);
        }

        function getProducts() {
            vm.products = cartService.getProductsInCart();
            refreshStatus();
        }

        function refreshStatus() {
            var total = 0;
            var checked = vm.products.length ? true : false;
            vm.isAnyChecked = false;
            vm.hasAnyProduct = vm.products.length > 0;
            angular.forEach(vm.products, function (product) {
                if (!product.statsInfo.completed) {
                    if (product.checked) {
                        total += product.quantity * product.price;
                        vm.isAnyChecked = true;
                    } else {
                        checked = false;
                    }
                }
            });
            vm.totalPrice = total;
            vm.isAllChecked = checked;
        }

        function onCheckout() {
            var selectProducts = [];
            var valid = true;
            angular.forEach(vm.products, function (product) {
                if (product.checked) {
                    if (product.statsInfo.completed) {
                        refreshStatus();
                        valid = false;
                    } else {
                        selectProducts.push(product);
                    }
                }
            });

            if (selectProducts.length > 0 && valid) {
                routerStateService.go('checkout', {'products': selectProducts});
            } else if (!valid) {
                alertService.autoShow('购物车中存在失效商品，请删除后重新结算');
            }
        }

        function goToProduct(item) {
            if (!item.statsInfo.completed) {
                routerStateService.go('product', {
                    id: item.id
                });
            }
        }
    }
})();
