(function () {
    'use strict';

    angular
        .module('app.order')
        .controller('CheckoutController', CheckoutController);

    CheckoutController.$inject = ['httpService', '$stateParams', '$scope', 'routerStateService', 'cartService'];

    function CheckoutController(httpService, $stateParams, $scope, routerStateService, cartService) {
        var vm = this;
        vm.address = null;
        vm.invoiceTitle = null;
        vm.canSubmit = false;
        vm.products = [];
        vm.totalPrice = 0;
        vm.subTotal = 0;
        vm.availablePoints = 0;
        vm.balance = 0;
        vm.pointsRate = 100;
        vm.usePoints = false;
        vm.showPoints = false;
        vm.goAddress = goAddress;
        vm.goInvoice = goInvoice;
        vm.submitOrder = submitOrder;
        vm.updateProduct = updateProduct;

        $scope.$on('checkout:refreshAddress', refreshAddress);
        $scope.$on('checkout:refreshInvoice', refreshInvoice);

        readInfo();

        //////////////////////////

        function goAddress() {
            var addressId = '';
            if (vm.address) {
                addressId = vm.address.id;
            }
            routerStateService.go('user-address-list', {
                address_id: addressId,
                products: vm.products
            });
        }

        function goInvoice() {
            routerStateService.go('custom-invoice');
        }

        function readInfo() {
            var promise = httpService.get('user/default_address');
            promise.then(function (result) {
                if (result.data.defaultAddress.id) {
                    vm.address = result.data.defaultAddress;
                }
                updateProduct();
            });
            httpService.get('get_balance').then(function (result) {
                vm.availablePoints = result.data.points;
                vm.pointsRate = result.data.rate;
                vm.balance = result.data.balance;
            });
        }

        function updateProduct(products) {
            vm.products = products || $stateParams.products || [];
            vm.canSubmit = vm.address != null;
            vm.subTotal = 0;
            angular.forEach(vm.products, function (product) {
                vm.subTotal += product.price * product.quantity;
                product.unreachable = true;
                if (vm.address) {
                    deliveryRange(product, vm.address.districtID);
                }
            });
            vm.totalPrice = vm.subTotal;
            if (vm.usePoints) {
                vm.totalPrice -= Math.min(vm.availablePoints / vm.pointsRate, vm.totalPrice);
            }
        }

        function deliveryRange(product, districtID) {
            var promise = httpService.get('product/' + product.productID +
            '/delivery_range/' + districtID);
            promise.then(function (result) {
                if (result.data.success &&
                    result.data.deliveryRange.deliveryAvailable) {
                    product.unreachable = false;
                } else {
                    vm.canSubmit = false;
                }
            });
        }

        function refreshAddress(event, address, products) {
            vm.address = address;
            $scope.$digest();
            updateProduct(products);
        }

        function refreshInvoice(event, title) {
            vm.invoiceTitle = title;
        }

        function submitOrder() {
            var sales = [];
            angular.forEach(vm.products, function (product) {
                sales.push({
                    sale: product.id,
                    quantity: product.quantity
                });
            });
            var promise = httpService.post('order', {
                address: vm.address.id,
                sales: sales,
                direct_buy: $stateParams.isDirectBuy,
                invoice_title: vm.invoiceTitle
            });
            promise.then(function (result) {
                cartService.refreshCart();
                routerStateService.go('pay', {transId: result.data.transId});
            });
        }
    }
})();
