(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('cartService', cartService);

    cartService.$inject = ['$rootScope', 'httpService'];

    function cartService($rootScope, httpService) {
        var productsData = null;
        var service = {
            addToCart: addToCart,
            bulkUpdateCart: bulkUpdateCart,
            flyToCart: flyToCart,
            getTotalQuantity: getTotalQuantity,
            getProductsInCart: getProductsInCart,
            updateCart: updateCart,
            removeFromCart: removeFromCart,
            bulkRemoveFromCart: bulkRemoveFromCart,
            refreshCart: refreshCart
        };
        return service;

        ////////////////

        function refreshCart(cbFunc) {
            var promise = httpService.get('cart');
            promise.then(function (result) {
                productsData = result.data;
                angular.forEach(productsData.sales, function (product) {
                    if (product.statsInfo.completed) {
                        product.checked = false;
                    }
                });
                $rootScope.$broadcast('cartService:refreshCountInCart');
                if (cbFunc && typeof cbFunc === 'function') {
                    cbFunc();
                }
            });
        }

        function getProductsInCart() {
            return productsData.sales;
        }

        function getTotalQuantity() {
            return productsData.totalQuantity;
        }

        function addToCart(id, quantity, cbFunc) {
            var promise = httpService.post('cart/add', {
                sale: id,
                action: 'add',
                quantity: quantity
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function updateCart(id, quantity, checked, cbFunc) {
            var promise = httpService.post('cart/add', {
                sale: id,
                action: 'update',
                quantity: quantity,
                checked: checked
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function bulkUpdateCart(products, cbFunc) {
            var promise = httpService.post('cart/bulk_update', {
                sales: products
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function removeFromCart(id, cbFunc) {
            var promise = httpService.post('cart/del', {
                sale: id
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function bulkRemoveFromCart(ids, cbFunc) {
            var promise = httpService.post('cart/bulk_delete', {
                sales: ids
            });
            promise.then(function (result) {
                if (result.data.success) {
                    refreshCart(cbFunc);
                }
            });
        }

        function flyToCart(productImgUrl) {
            event.cancelBubble = true;
            var cartTab = $('a.cart-tab');
            var cartTabPosition = cartTab[0].getBoundingClientRect();
            var startPosition = {
                x: 0,
                y: 0
            };
            var endPosition = {
                x: cartTabPosition.left - event.x + cartTab.width() / 2,
                y: cartTabPosition.top - event.y
            };
            var controlPosition = {
                x: endPosition.x,
                y: 0
            };
            var fakeProduct = cloneFlyingProduct(event.x, event.y, productImgUrl);
            fly(startPosition, endPosition, controlPosition, 0, 0, fakeProduct);
        }

        function cloneFlyingProduct(x, y, imgURL) {
            var fakeProduct = $('<div class="product-shadow"><img src=' + imgURL + '></div>').css({
                'left': x + 'px',
                'top': y + 'px'
            });
            $(document.body).append(fakeProduct);
            return fakeProduct;
        }

        function fly(sp, ep, cp, i, factor, fakeProduct) {
            if (i <= 100) {
                var step = twoBessel(sp, ep, cp, i);
                fakeProduct.css('transform', 'translate(' + step.x + 'px, ' + step.y + 'px)');
                i += factor += 0.15;
                setTimeout(function () {
                    fly(sp, ep, cp, i, factor, fakeProduct);
                }, 15);
            } else {
                fakeProduct.remove();
            }
        }

        function twoBessel(sp, ep, cp, step) {
            var t = step / 100;
            var a = t * t;
            var b = 2 * t * (1 - t);
            var c = Math.pow(1 - t, 2);
            return {
                x: c * sp.x + b * cp.x + a * ep.x,
                y: c * sp.y + b * cp.y + a * ep.y
            };
        }
    }
})();
