(function() {
    'use strict';

    angular
        .module('app.service')
        .factory('httpInterceptor', httpInterceptor);

    httpInterceptor.$inject = ['$rootScope', '$q', 'authService'];

    function httpInterceptor($rootScope, $q, authService) {
        var service = {
            request: requestHandler,
            requestError: requestErrorHandler,
            response: responseHandler,
            responseError: responseErrorHandler

        };
        return service;

        ////////////////

        function requestHandler(config) {
            if (config.responseType === 'json') {
                $rootScope.$broadcast('loading:show');
            }
            return config;
        }

        function requestErrorHandler(rejection) {
            return $q.reject(rejection);
        }

        function responseHandler(response) {
            $rootScope.$broadcast('loading:hide');
            if (response.data.error === undefined) {
                return response;
            } else {
                $rootScope.$broadcast('error:show', response.data.error);
                return $q.reject(response);
            }
        }

        function responseErrorHandler(rejection) {
            $rootScope.$broadcast('loading:hide');
            // TODO: Log error
            var message = '服务器错误，请稍后重试';
            var nextState = null;
            if (rejection.status === 400) {
                message = '错误请求';
            } else if (rejection.status === 401) {
                authService.signOut();
                message = '会话已过期，请重新登录';
                nextState = 'login';
            } else if (rejection.status === 404) {
                message = '所访问资源不存在';
            } else if (rejection.data && rejection.data.error) {
                message = rejection.data.error.message;
            }
            $rootScope.$broadcast('error:show', {
                code: rejection.status,
                message: message,
                nextState: nextState
            });
            return $q.reject(rejection);
        }
    }
})();
