(function () {
    'use strict';

    angular
        .module('app.service')
        .factory('modalManagerService', modalManagerService);

    modalManagerService.$inject = ['$ionicModal'];


    function modalManagerService($ionicModal) {
        var modalList = {
            about: {
                templateUrl: 'templates/partials/about.modal.html'
            },
            terms: {
                templateUrl: 'templates/partials/terms.modal.html'
            },
            orderReview: {
                templateUrl: 'templates/partials/order-review.modal.html'
            },
            productDeliveryArea: {
                templateUrl: 'templates/partials/delivery-list.modal.html',
                animation: 'slide-in-left',
                onShown: function (modal) {
                    modal.scope.$broadcast('deliveryArea:show', modal);
                }
            },
            userDeliveryArea: {
                templateUrl: 'templates/partials/delivery-list.modal.html',
                onShown: function (modal) {
                    modal.scope.$broadcast('deliveryArea:show', modal);
                }
            }
        };
        var service = {
            addModal: addModal,
            getModal: getModal,
            showModal: showModal,
            closeModalByName: closeModalByName,
            closeModal: closeModal
        };
        return service;

        ////////////////

        function getModalName(name) {
            return name + 'Modal';
        }

        function addModal(name, scope, options, show_immediately) {
            if (!(name && name.trim())) {
                return;
            }
            var modal = getModal(name, scope);
            if (modal) {
                if (show_immediately) {
                    modal.show();
                }
                return;
            }
            var modalOptions = angular.extend({}, modalList[name] || {}, options || {});
            $ionicModal.fromTemplateUrl(modalOptions.templateUrl, {
                scope: scope,
                animation: modalOptions.animation || 'slide-in-up'
            }).then(function (modal) {
                var modalName = getModalName(name);
                scope[modalName] = modal;
                modal.$el.addClass(modalName.replace(/([A-Z])/g, '-$1').toLowerCase());
                modal.$el.css('background', 'none');
                if (show_immediately) {
                    modal.show();
                }
            });

            scope.$on('$destroy', function () {
                var modal = getModal(name, scope);
                if (modal) {
                    modal.remove();
                }
            });

            scope.$on('modal.shown', function(event, modal) {
                console.info(getModalName(name) + ' is shown!');

                if (modalOptions.onShown) {
                    modalOptions.onShown(modal);
                }
            });

            scope.$on('modal.hidden', function(event, modal) {
                console.info(getModalName(name) + ' is hidden!');

                if (modalOptions.onHidden) {
                    modalOptions.onHidden(modal);
                }
            });

            scope.$on('modal.removed', function() {
                console.info(getModalName(name) + ' is removed!');
            });

        }

        function getModal(name, scope) {
            return scope[getModalName(name)];
        }

        function showModal(name, scope, options) {
            addModal(name, scope, options, true);
        }

        function closeModalByName(name, scope, onClose) {
            closeModal(getModal(name, scope), onClose);
        }

        function closeModal(modal, onClose) {
            if (modal) {
                if (onClose) {
                    onClose(modal);
                }
                modal.hide();
            }
        }
    }
})();
