(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('BindStrategyController', BindStrategyController);

    BindStrategyController.$inject = ['$scope', 'httpService', 'alertService',
        '$stateParams', '$ionicHistory', 'authService'];

    function BindStrategyController($scope, httpService, alertService,
                                    $stateParams, $ionicHistory, authService) {
        var vm = this;
        var regExpPhone = /^[0-9]{11}$/;
        var regExpEmail = /^.+@[^.].*.[a-z]{2,10}$/;

        vm.code = '';
        vm.btnText = '';
        vm.title = '';
        vm.type = '';
        vm.typeText = '';
        vm.typeValue = '';
        vm.isBind = true;
        vm.fieldCode = '获取验证码';
        vm.phone_error_text = '手机号格式不正确!';
        vm.email_error_text = '邮箱格式不正确!';
        vm.code_error_text = '验证码不能为空!';
        vm.flag = false;
        vm.getCode = getCode;
        vm.bind = bind;

        init();

        function init() {
            if ($stateParams.isBind) {
                if ($stateParams.type === 'phone') {
                    vm.title = '绑定手机号';
                    vm.typeText = '手机号';
                } else if ($stateParams.type === 'email') {
                    vm.title = '绑定邮箱';
                    vm.typeText = '邮箱';
                }
                vm.btnText = '绑定';
            } else {
                if ($stateParams.type === 'phone') {
                    vm.title = '解绑手机号';
                    vm.typeText = '手机号';
                } else if ($stateParams.type === 'email') {
                    vm.title = '解绑邮箱';
                    vm.typeText = '邮箱';
                }
                vm.typeValue = $stateParams.typeValue;
                vm.btnText = '解绑';
            }
            vm.isBind = $stateParams.isBind;
            vm.type = $stateParams.type;
        }

        function bind() {
            var url;
            if (validate()) {
                if ($stateParams.type === 'phone') {
                    url = vm.isBind ? 'user/bind/mobile' : 'user/unbind/mobile';
                } else if ($stateParams.type === 'email') {
                    url = vm.isBind ? 'user/bind/email' : 'user/unbind/email';
                }
                httpService.post(url, {
                    address: vm.typeValue,
                    verify_code: vm.code
                }).then(function (result) {
                    if (result.data.success) {
                        updateUser();
                        $ionicHistory.goBack();
                    } else if (result.data.errorResponse) {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function updateUser() {
            var user = authService.getAuthenticatedUser();
            if ($stateParams.isBind) {
                if ($stateParams.type === 'phone') {
                    user.mobile = vm.typeValue;
                } else if ($stateParams.type === 'email') {
                    user.email = vm.typeValue;
                }
            } else {
                if ($stateParams.type === 'phone') {
                    user.mobile = '';
                } else if ($stateParams.type === 'email') {
                    user.email = '';
                }
            }
            authService.updateUser(user);
        }

        function validate() {
            if (vm.isBind) {
                if ($stateParams.type === 'phone') {
                    if (!regExpPhone.test(vm.typeValue)) {
                        showPrompt(vm.phone_error_text);
                        return false;
                    }
                } else if ($stateParams.type === 'email') {
                    if (!regExpEmail.test(vm.typeValue)) {
                        showPrompt(vm.email_error_text);
                        return false;
                    }
                }
            } else {
                if ($.trim(vm.code) === '') {
                    showPrompt(vm.code_error_text);
                    return false;
                }
            }

            return true;
        }

        function getCode() {
            var url, regExp, info, address, need_user, parms;
            if ($stateParams.type === 'phone') {
                url = 'verification/mobile';
                address = vm.typeValue;
                need_user = true;
                if (vm.isBind) {
                    need_user = false;
                    regExp = regExpPhone;
                    info = vm.phone_error_text;
                }
            } else if ($stateParams.type === 'email') {
                url = 'verification/email';
                address = vm.typeValue;
                need_user = true;
                if (vm.isBind) {
                    need_user = false;
                    regExp = regExpEmail;
                    info = vm.email_error_text;
                }
            }
            if ( (regExp && regExp.test(address)) || !vm.isBind) {
                if (need_user) {
                    parms = {
                        address: address,
                        need_user: need_user
                    };
                } else {
                    parms = {
                        address: address
                    };
                }
                httpService.post(url, parms).then(function (result) {
                    if (result.data.success) {
                        var count = result.data.resendInterval;
                        vm.fieldCode = count + '秒后' + vm.fieldCode;
                        vm.flag = !vm.flag;
                        var timer = setInterval(function () {
                            if (count < 1) {
                                vm.fieldCode = '获取验证码';
                                vm.flag = !vm.flag;
                                $scope.$digest();
                                clearInterval(timer);
                                return;
                            }
                            count -= 1;
                            vm.fieldCode = count + '秒后获取验证码';
                            $scope.$digest();
                        }, 1000);
                    } else if (result.data.errorResponse) {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            } else {
                showPrompt(info);
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();
