(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('ChangePasswordController', ChangePasswordController);

    ChangePasswordController.$inject = ['alertService', '$stateParams', 'httpService', '$ionicHistory'];

    function ChangePasswordController(alertService, $stateParams, httpService, $ionicHistory) {
        var vm = this;
        vm.oldPassword = '';
        vm.newPassword = '';
        vm.confirmPassword = '';
        vm.onChangePassword = onChangePassword;
        vm.onResetPassword = onResetPassword;

        ////////////////

        function onChangePassword() {
            if (validate(true)) {
                httpService.post('user/reset_password', {
                    'old_password': vm.oldPassword,
                    'new_password': vm.newPassword,
                    're_password': vm.confirmPassword
                }).then(function (result) {
                    if (result.data.success) {
                        $ionicHistory.goBack(-2);
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function onResetPassword() {
            if (validate(false)) {
                httpService.post('user/reset_password', {
                    token: $stateParams.token,
                    new_password: vm.newPassword,
                    re_password: vm.confirmPassword
                }).then(function (result) {
                    if (result.data.success) {
                        $ionicHistory.goBack(-2);
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function validate(need_old_pw) {
            var is_ok = false;
            if (need_old_pw && vm.oldPassword.length <= 0) {
                showPrompt('请输入当前密码')
            } else if (vm.newPassword.length <= 0) {
                showPrompt('请输入新密码')
            } else if (!/^[^\n\r]{6,20}$/.test(vm.newPassword)) {
                showPrompt('新密码长度不符合要求')
            } else if (vm.confirmPassword.length <= 0) {
                showPrompt('请再次输入新密码')
            } else if (vm.newPassword !== vm.confirmPassword) {
                showPrompt('两次输入的密码不一样')
            } else {
                is_ok = true;
            }
            return is_ok;
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();

