(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('LoginController', LoginController);

    LoginController.$inject = ['alertService', '$timeout', '$ionicHistory', 'httpService', 'authService', 'routerStateService'];

    function LoginController(alertService, $timeout, $ionicHistory, httpService, authService, routerStateService) {
        var vm = this;
        vm.onLogin = onLogin;
        vm.onRegister = onRegister;
        vm.onForgotPassword = onForgotPassword;
        vm.password = '';
        vm.userName = '';

        ////////////////

        function onLogin() {
            var lenUserName = vm.userName.replace(/(^s*)|(s*$)/g, '').length;
            var lenPassword = vm.password.replace(/(^s*)|(s*$)/g, '').length;

            if (lenUserName === 0 && lenPassword === 0) {
                showPrompt('请输入用户名和密码');
            } else if (lenUserName === 0) {
                showPrompt('请输入用户名');
            } else if (lenPassword === 0) {
                showPrompt('请输入密码');
            } else {
                var reqData = {
                    username: vm.userName,
                    password: vm.password,
                    remember: true
                };
                var promise = httpService.post('signin', reqData);
                promise.then(function (result) {
                    if (result.data.success) {
                        authService.signIn(result.data.userInfo);
                        $ionicHistory.goBack();
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }

        function onRegister() {
            routerStateService.go('register');
        }

        function onForgotPassword() {
            routerStateService.go('retrieve-password');
        }
    }

})();
