(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('OrderListController', OrderListController);

    OrderListController.$inject = ['$scope', 'httpService', 'routerStateService', 'appConst', 'modalManagerService'];

    function OrderListController($scope, httpService, routerStateService, appConst, modalManagerService) {
        var vm = this;
        vm.getOrderByCondition = getOrderByCondition;
        vm.hasMore = true;
        vm.onReview = onReview;
        vm.onPaying = onPaying;
        vm.onOrderInfo = onOrderInfo;
        vm.orderList = [];
        vm.page = 0;
        $scope.appConst = appConst;

        ////////////////

        function onPaying(order) {
            routerStateService.go('pay', {transId: order.trans_id});
        }

        function onReview(order) {
            $scope.order = order;
            $scope.reload = false;
            modalManagerService.showModal('orderReview', $scope, {
                onHidden: function (modal) {
                    if (modal.scope.reload) {
                        order.reviewed = true;
                        routerStateService.reload();
                    }
                }
            });
        }

        function onOrderInfo(order) {
            routerStateService.go('order-info', {orderId: order.id});
        }

        function getOrderByCondition() {
            if (!vm.hasMore) {
                return;
            }

            var orderList = httpService.get('order_list', {page: vm.page});
            orderList.then(function (result) {
                vm.orderList = vm.orderList.concat(result.data.orderList);
                if (result.data.orderList.length === 0) {
                    vm.hasMore = false;
                } else {
                    vm.page++;
                }
            }).catch(function () {
                vm.hasMore = false;
            }).finally(function () {
                $scope.$broadcast('scroll.infiniteScrollComplete');
            });
        }
    }
})();
