(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('RegisterController', RegisterController);

    RegisterController.$inject = ['alertService', '$scope', 'httpService', '$ionicHistory', 'authService', 'modalManagerService', 'routerStateService'];

    function RegisterController(alertService, $scope, httpService, $ionicHistory, authService, modalManagerService, routerStateService) {
        var vm = this;
        vm.username = '';
        vm.password = '';
        vm.repeatPassword = '';
        vm.phone = '';
        vm.code = '';
        vm.second = '';
        vm.disable = false;
        vm.flag = false;

        vm.back = back;
        vm.register = register;
        vm.onLogin = onLogin;
        vm.showTermsModal = showTermsModal;
        vm.getCode = getCode;
        vm.finish = finish;

        ////////////////

        function back() {
            if (vm.flag) {
                vm.flag = false;
            } else {
                $ionicHistory.goBack();
            }
        }

        function register() {
            var flag = validate();
            if (flag) {
                var promise = httpService.post('signup', {
                    username: vm.username,
                    password: vm.password,
                    repassword: vm.repeatPassword,
                    pre_signup: true
                });
                promise.then(function (result) {
                    if (result.data.success) {
                        vm.flag = true;
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }

        function validate() {
            if (vm.username.replace(/^\s+|\s+$/g, '') == '') {
                showPrompt('用户名不能为空!');
                return false;
            } else if (!/^[0-9|a-z|A-Z|_]{4,20}$/.test(vm.username)) {
                showPrompt('请输入正确的用户名!');
                return false;
            }

            if (vm.password == '') {
                showPrompt('密码不能为空!');
                return false;
            } else if (!/^[^\n\r]{6,20}$/.test(vm.password)) {
                showPrompt('密码长度不符合要求!');
                return false;
            }

            if (vm.repeatPassword == '') {
                showPrompt('确认密码不能为空!');
                return false;
            } else if (vm.password != vm.repeatPassword) {
                showPrompt('两次输入的密码不一致!');
                return false;
            }

            return true;
        }

        function validatePhone() {
            if (!/^[0-9]{11}$/.test(vm.phone)) {
                showPrompt('请输入正确的手机号!');
                return false;
            }
            return true;
        }

        function validateCode() {
            if ($.trim(vm.code) == '') {
                showPrompt('验证码不能为空!');
                return false;
            }
            return true;
        }

        function getCode() {
            if (validatePhone()) {
                vm.disable = true;
                getServerCode(function (soconds) {
                    var s = soconds;

                    vm.second = '（' + s + '）';
                    var timer = setInterval(function () {
                        s--;
                        if (s < 0) {
                            clearInterval(timer);
                            vm.second = '';
                            vm.disable = false;
                        } else {
                            vm.second = '（' + s + '）';
                        }
                        $scope.$digest();

                    }, 1000);
                });
            }
        }

        function getServerCode(callback) {
            httpService.post('verification/mobile', {
                address: vm.phone
            }).then(function (result) {
                if (result.data.success) {
                    callback(result.data.resendInterval);
                } else {
                    vm.disable = false;
                    showPrompt(result.data.errorResponse.errMsg);
                }
            });
        }

        function finish() {
            if (validatePhone() && validateCode()) {
                httpService.post('register', {
                    username: vm.username,
                    password: vm.password,
                    repassword: vm.repeatPassword,
                    media: 'mobile',
                    address: vm.phone,
                    keycode: vm.code
                }).then(function (result) {
                    if (result.data.success) {
                        authService.signIn(result.data.userInfo);
                        $ionicHistory.goBack();
                    } else {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            }
        }

        function showTermsModal() {
            modalManagerService.showModal('terms', $scope);
        }

        function onLogin() {
            routerStateService.go('login');
        }
    }

})();
