(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('RetrievePasswordController', RetrievePasswordController);

    RetrievePasswordController.$inject = ['$scope', 'httpService', 'alertService', '$timeout', 'routerStateService'];

    function RetrievePasswordController($scope, httpService, alertService, $timeout, routerStateService) {
        var vm = this;
        var timer = null;
        var regExpPhone = /^[0-9]{11}$/;
        var regExpEmail = /^.+@[^.].*.[a-z]{2,10}$/;

        vm.type = 'phone';
        vm.phone = '';
        vm.phoneCode = '';
        vm.email = '';
        vm.emailCode = '';
        vm.change = change;
        vm.flag = false;
        vm.second = '';
        vm.retrievePhone = retrievePhone;
        vm.retrieveEmail = retrieveEmail;
        vm.getPhoneCode = getPhoneCode;
        vm.getEmailCode = getEmailCode;

        function change(type) {
            vm.type = type;
            vm.flag = false;
            clearInterval(timer);
            vm.second = '';
        }

        function retrievePhone() {
            if (validatePhone()) {
                retrieve({
                    url: 'user/forgot_password/mobile',
                    address: vm.phone,
                    verify_code: vm.phoneCode
                });
            }
        }

        function retrieveEmail() {
            if (validateEmail()) {
                retrieve({
                    url: 'user/forgot_password/email',
                    address: vm.email,
                    verify_code: vm.emailCode
                });
            }
        }

        function retrieve(ops) {
            httpService.post(ops.url, {
                address: ops.address,
                verify_code: ops.verify_code
            }).then(function (result) {
                if (result.data.success) {
                    routerStateService.go('reset-password', {token: result.data.token});
                } else if (result.data.errorResponse) {
                    showPrompt(result.data.errorResponse.errMsg);
                }
            });
        }

        function validatePhone() {
            if (!regExpPhone.test($.trim(vm.phone))) {
                showPrompt('请输入正确的手机号码！');
                return false;
            }

            return true;
        }

        function validateEmail() {
            if (!regExpEmail.test($.trim(vm.email))) {
                showPrompt('请输入正确的邮箱！');
                return false;
            }

            return true;
        }

        function getPhoneCode() {
            getCode({
                validate: regExpPhone,
                url: 'verification/mobile',
                address: vm.phone
            });
        }

        function getEmailCode() {
            getCode({
                validate: regExpEmail,
                url: 'verification/email',
                address: vm.email
            });
        }

        function getCode(ops) {
            if (ops.validate.test($.trim(ops.address))) {
                vm.flag = true;
                httpService.post(ops.url, {
                    address: ops.address,
                    need_user: true
                }).then(function (result) {
                    if (result.data.success) {
                        var s = result.data.resendInterval;

                        vm.second = '（' + s + '）';
                        timer = setInterval(function () {
                            s--;
                            if (s < 0) {
                                clearInterval(timer);
                                vm.second = '';
                                vm.flag = false;
                            } else {
                                vm.second = '（' + s + '）';
                            }
                            $scope.$digest();

                        }, 1000);
                    } else if (result.data.errorResponse) {
                        showPrompt(result.data.errorResponse.errMsg);
                    }
                });
            } else {
                if (vm.type === 'phone') {
                    showPrompt('请输入正确的手机号码！');
                } else if (vm.type === 'email') {
                    showPrompt('请输入正确的邮箱！');
                }
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();
