(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('UserAddressListController', UserAddressListController);

    UserAddressListController.$inject = ['httpService', 'alertService', '$timeout', 'routerStateService', '$stateParams', '$ionicHistory', '$rootScope'];

    function UserAddressListController(httpService, alertService, $timeout, routerStateService, $stateParams, $ionicHistory, $rootScope) {
        var vm = this;
        var products = null;
        vm.addressList = [];
        vm.select = select;
        vm.deleteAddress = deleteAddress;
        vm.updateAddress = updateAddress;
        vm.updateDefaultAddress = updateDefaultAddress;
        vm.createAddress = createAddress;
        vm.currentIsDefault = null;
        vm.defaultAddress = {};
        vm.showSetDefaultBtn = $stateParams.address_id == null;
        fetchAddressList();

        function fetchAddressList() {
            var promise = httpService.get('user/address_list');
            promise.then(function (result) {
                vm.addressList = result.data.addressList || [];
                if (vm.showSetDefaultBtn) {
                    angular.forEach(vm.addressList, function (item) {
                        if (item.isDefault) {
                            vm.defaultAddress = item;
                        }
                    });
                } else {
                    products = $stateParams.products;
                    angular.forEach(vm.addressList, function (item) {
                        console.debug($stateParams.address_id);
                        console.debug(item.id);
                        console.debug('~~~~~');
                        if (item.isDefault) {
                            vm.defaultAddress = item;
                            item.__isSelect__ = true;
                            if ($stateParams.address_id === '' || $stateParams.address_id === item.id) {
                                item.__isSelect__ = true;
                            } else {
                                item.__isSelect__ = false;
                            }
                        } else if ($stateParams.address_id === item.id) {
                            item.__isSelect__ = true;
                        }
                    });
                }
            });
        }

        function createAddress() {
            routerStateService.go('user-address', {address: null});
        }

        function select(item) {
            if (!vm.showSetDefaultBtn) {
                $ionicHistory.goBack();
                $timeout(function () {
                    $rootScope.$broadcast('checkout:refreshAddress', item, products);
                }, 10);
            } else {
                vm.currentIsDefault = item;
                updateDefaultAddress();
            }
        }

        function deleteAddress(item) {
            if (vm.defaultAddress.id == item.id) {
                showPrompt('默认地址不能删除!');
                return;
            }
            if (vm.currentIsDefault && vm.currentIsDefault.id === item.id) {
                vm.currentIsDefault = null;
            }
            var promise = httpService.post('user/del_address/' + item.id);
            promise.then(function (result) {
                if (result.data.success) {
                    fetchAddressList();
                } else if (result.data.errorRespones) {
                    showPrompt(result.data.errorRespones.errMsg);
                }
            });
        }

        function updateAddress(item) {
            routerStateService.go('user-address', {address: item});
        }

        function updateDefaultAddress() {
            if (vm.currentIsDefault) {
                var promise = httpService.post('user/set_default_address/' + vm.currentIsDefault.id);
                promise.then(function (result) {
                    if (result.data.success) {
                        vm.defaultAddress.isDefault = false;
                        vm.currentIsDefault.isDefault = true;
                        vm.defaultAddress = vm.currentIsDefault;
                        showPrompt('设置默认地址成功!');
                    } else if (result.data.errorRespones) {
                        showPrompt(result.data.errorRespones.errMsg);
                    }
                });
            } else {
                showPrompt('请选择你要设置的默认地址!');
            }
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();
