(function () {
    'use strict';

    angular
        .module('app.user')
        .controller('UserAddressController', UserAddressController);

    UserAddressController.$inject = [
        '$scope', 'httpService', 'alertService', '$rootScope', '$timeout',
        '$ionicHistory', '$stateParams', 'modalManagerService'
    ];

    function UserAddressController($scope, httpService, alertService, $rootScope,
                                   $timeout, $ionicHistory, $stateParams, modalManagerService) {
        var vm = this;
        vm.address = $stateParams.address;
        vm.title = $stateParams.address != null ? '编辑收货地址' : '新建收货地址';
        vm.onDeliveryAreaClick = onDeliveryAreaClick;
        vm.saveAddress = saveAddress;

        ////////////////

        function onDeliveryAreaClick() {
            if (vm.address) {
                $scope.defaultArea = {
                    province: {
                        id: vm.address.provinceID,
                        name: vm.address.province
                    },
                    city: {
                        id: vm.address.cityID,
                        name: vm.address.city
                    },
                    district: {
                        id: vm.address.districtID,
                        name: vm.address.district
                    }
                };
            } else {
                $scope.defaultArea = null;
            }
            $scope.selectedArea = null;
            $scope.modalTitle = '配送所在地区';
            modalManagerService.showModal('userDeliveryArea', $scope, {
                onHidden: function (modal) {
                    if (!modal.scope.selectedArea) {
                        return;
                    }
                    var area = modal.scope.selectedArea;
                    vm.address = angular.extend({}, vm.address, {
                        districtID: area.district.id,
                        district: area.district.name,
                        cityID: area.city.id,
                        city: area.city.name,
                        provinceID: area.province.id,
                        province: area.province.name
                    });
                    modal.scope.selectedArea = null;
                }
            });
        }

        function saveAddress() {
            var promise = null;
            if (validateAddress()) {
                if ($stateParams.address) {
                    promise = httpService.post('user/edit_address/' + vm.address.id, {
                        address_id: vm.address.id,
                        recipient: vm.address.recipient,
                        phone: vm.address.phone,
                        district_id: vm.address.districtID,
                        address_desc: vm.address.addressDesc,
                        is_default: vm.address.isDefault
                    });
                } else {
                    promise = httpService.post('user/add_address', {
                        recipient: vm.address.recipient,
                        phone: vm.address.phone,
                        district_id: vm.address.districtID,
                        address_desc: vm.address.addressDesc
                    });
                }
                promise.then(function (result) {
                    if (!$stateParams.address && $stateParams.type) {
                        if (result.data.success) {
                            httpService.get('user/address_list')
                                .then(function (obj) {
                                    var addressList = obj.data.addressList || [];
                                    $.each(addressList, function (index, item) {
                                        if (result.data.userAddress.id === item.id) {
                                            $ionicHistory.goBack(-2);
                                            $timeout(function () {
                                                $rootScope.$broadcast('checkout:refreshAddress', item);
                                            }, 10);
                                        }
                                    });
                                });
                        } else {
                            showPrompt(result.data.errorRespones.errMsg);
                        }
                    } else {
                        if (result.data.success) {
                            $ionicHistory.goBack();
                        } else if (result.data.errorRespones) {
                            showPrompt(result.data.errorRespones.errMsg);
                        }
                    }
                });
            }
        }

        function validateAddress() {
            if ($.trim(vm.address.recipient) === '') {
                showPrompt('收件人不能为空!');
                return false;
            }
            if ($.trim(vm.address.phone) === '') {
                showPrompt('联系方式不能为空！');
                return false;
            }
            if (!vm.address.districtID) {
                showPrompt('请选择所在地区!');
                return false;
            }
            if ($.trim(vm.address.addressDesc) === '') {
                showPrompt('详细地址不能为空!');
                return false;
            }
            return true;
        }

        function showPrompt(msg) {
            alertService.autoShow(msg);
        }
    }
})();
