(function() {
    'use strict';

    angular
        .module('app.user')
        .controller('UserInfoController', UserInfoController);

    UserInfoController.$inject = ['httpService', 'appConst'];

    function UserInfoController(httpService, appConst) {
        var vm = this;
        vm.birth = null;
        vm.gender = null;
        vm.genderText = '性别';
        vm.genderOptions = [];
        vm.lastBirth = null;
        vm.lastGender = null;
        vm.updateInfo = updateInfo;
        vm.change = change;

        readInfo();
        ////////////////

        function readInfo() {
            vm.genderOptions = [{
                id: appConst.USER_GENDER.UNKNOWN,
                label: '保密'
            }, {
                id: appConst.USER_GENDER.MALE,
                label: '男'
            }, {
                id: appConst.USER_GENDER.FEMALE,
                label: '女'
            }];

            var user_profile = httpService.get('user/profile');
            user_profile.then(function(result) {
                var user_info = result.data.userProfile.user_info;
                if (user_info) {
                    vm.gender = vm.lastGender = user_info.gender;
                    vm.birth = vm.lastBirth = new Date(user_info.birth);
                }
                change();
            });
        }


        function updateInfo() {
            if (vm.lastBirth === vm.birth && vm.lastGender === vm.gender){
                return;
            }

            var reqData = {};
            if (vm.birth !== null) {
                reqData.birth = vm.birth.getFullYear() + '-' +
                    (vm.birth.getMonth() + 1) + '-' + vm.birth.getDate();
            }
            if (vm.gender !== null) {
                reqData.gender = vm.gender;
            }
            var promise = httpService.post('user/edit_profile', reqData);
            promise.then(function(result) {
                if (result.data.success === true) {
                    vm.lastBirth = vm.birth;
                    vm.lastGender = vm.gender;
                }
            });
        }

        function change() {
            if (vm.gender === 0) {
                vm.genderText = '保密';
            } else if (vm.gender === 1) {
                vm.genderText = '男';
            } else if (vm.gender === 2) {
                vm.genderText = '女';
            } else {
                vm.genderText = '性别';
            }
        }
    }
})();
