(function () {
    'use strict';

    angular
        .module('app.utils')
        .controller('DeliveryAreaListController', DeliveryAreaListController);

    DeliveryAreaListController.$inject = ['httpService', '$scope', 'modalManagerService'];

    function DeliveryAreaListController(httpService, $scope, modalManagerService) {
        var vm = this;
        vm.areaList = [];
        vm.goBack = goBack;
        vm.onAreaSelected = onAreaSelected;
        vm.closeModal = closeModal;
        vm.selectedAreaID = null;
        vm.showType = 'province';
        vm.showBackBtn = vm.showType === 'city' || vm.showType === 'district';

        $scope.$on('deliveryArea:show', function(event, modal) {
            vm.defaultArea = modal.scope.defaultArea;
            vm.selectedArea = modal.scope.selectedArea;
            vm.title = modal.scope.modalTitle;
            vm.modal = modal;

            fetchProvinceList();
        });

        ////////////////

        function fetchCityList(province) {
            var promise = httpService.get('city_list/' + province.id);
            promise.then(function (result) {
                var cityList = result.data.cityList;
                vm.areaList = [];
                angular.forEach(cityList, function (city) {
                    vm.areaList.push({
                        id: city.cityID,
                        name: city.cityName
                    });
                });
                vm.showType = 'city';
                var city = vm.areaList[0];
                if (vm.defaultArea && vm.defaultArea.province && province.id === vm.defaultArea.province.id) {
                    city = vm.defaultArea.city;
                }
                if (vm.selectedArea && vm.selectedArea.city && province.id === vm.selectedArea.province.id) {
                    city = vm.selectedArea.city;
                }
                if (vm.selectedArea) {
                    vm.selectedArea.province = province;
                } else {
                    vm.selectedArea = {
                        province: province
                    };
                }
                vm.selectedAreaID = city.id;
            });
        }

        function fetchDistrictList(city) {
            var promise = httpService.get('district_list/' + city.id);
            promise.then(function (result) {
                var districtList = result.data.districtList;
                vm.areaList = [];
                angular.forEach(districtList, function(district) {
                    vm.areaList.push({
                        id: district.districtID,
                        name: district.districtName
                    });
                });
                vm.showType = 'district';
                vm.selectedArea.city = city;
                var district = vm.areaList[0];
                if (vm.defaultArea && vm.defaultArea.city && city.id === vm.defaultArea.city.id) {
                    district = vm.defaultArea.district;
                }
                vm.selectedAreaID = district.id;
            });
        }

        function fetchProvinceList() {
            var promise = httpService.get('province_list');
            promise.then(function (result) {
                var provinceList = result.data.provinceList;
                vm.areaList = [];
                angular.forEach(provinceList, function (province) {
                    vm.areaList.push({
                        id: province.provinceID,
                        name: province.provinceName
                    });
                });
                vm.showType = 'province';
                var province = vm.areaList[0];
                if (vm.defaultArea && vm.defaultArea.province.id) {
                    province = vm.defaultArea.province;
                }
                if (vm.selectedArea && vm.selectedArea.province) {
                    province = vm.selectedArea.province;
                }
                vm.selectedAreaID = province.id;
            });
        }

        function onAreaSelected(area) {
            if (vm.showType === 'province') {
                fetchCityList(area);
            } else if (vm.showType === 'city') {
                fetchDistrictList(area);
            } else if (vm.showType === 'district') {
                modalManagerService.closeModal(vm.modal, function(modal) {
                    vm.selectedArea.district = area;
                    modal.scope.selectedArea = vm.selectedArea;
                });
            }
        }

        function goBack() {
            if (vm.showType === 'city') {
                fetchProvinceList();
            } else if (vm.showType === 'district') {
                fetchCityList(vm.selectedArea.province);
            }
        }

        function closeModal() {
            modalManagerService.closeModal(vm.modal);
        }
    }
})();
